@@ -34,7 +34,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: amrr.c 1721 2006-09-20 08:45:13Z mentor $
+ * $Id: amrr.c 2814 2007-10-31 05:39:41Z proski $
  */
 
 /*
@@ -62,9 +62,10 @@
 
 #include <net80211/if_media.h>
 #include <net80211/ieee80211_var.h>
+#include <net80211/ieee80211_rate.h>
 
-#include "if_athrate.h"
 #include "if_athvar.h"
+#include "if_ath_hal.h"
 #include "ah_desc.h"
 
 #include "amrr.h"
@@ -88,24 +89,22 @@ static void ath_rate_update(struct ath_softc *, struct ieee80211_node *, int);
 static void ath_rate_ctl_start(struct ath_softc *, struct ieee80211_node *);
 static void ath_rate_ctl(void *, struct ieee80211_node *);
 
-void
+static void
 ath_rate_node_init(struct ath_softc *sc, struct ath_node *an)
 {
 	/* NB: assumed to be zero'd by caller */
 	ath_rate_update(sc, &an->an_node, 0);
 }
-EXPORT_SYMBOL(ath_rate_node_init);
 
-void
+static void
 ath_rate_node_cleanup(struct ath_softc *sc, struct ath_node *an)
 {
 }
-EXPORT_SYMBOL(ath_rate_node_cleanup);
 
-void
+static void
 ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 	int shortPreamble, size_t frameLen,
-	u_int8_t *rix, int *try0, u_int8_t *txrate)
+	u_int8_t *rix, unsigned int *try0, u_int8_t *txrate)
 {
 	struct amrr_node *amn = ATH_NODE_AMRR(an);
 
@@ -116,29 +115,29 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 	else
 		*txrate = amn->amn_tx_rate0;
 }
-EXPORT_SYMBOL(ath_rate_findrate);
 
-void
-ath_rate_setupxtxdesc(struct ath_softc *sc, struct ath_node *an,
-	struct ath_desc *ds, int shortPreamble, size_t frame_size, u_int8_t rix)
+static void
+ath_rate_get_mrr(struct ath_softc *sc, struct ath_node *an, int shortPreamble,
+		 size_t frame_size, u_int8_t rix, struct ieee80211_mrr *mrr)
 {
 	struct amrr_node *amn = ATH_NODE_AMRR(an);
 
-	ath_hal_setupxtxdesc(sc->sc_ah, ds
-		, amn->amn_tx_rate1sp, amn->amn_tx_try1	/* series 1 */
-		, amn->amn_tx_rate2sp, amn->amn_tx_try2	/* series 2 */
-		, amn->amn_tx_rate3sp, amn->amn_tx_try3	/* series 3 */
-	);
+	mrr->rate1 = amn->amn_tx_rate1sp;
+	mrr->retries1 = amn->amn_tx_try1;
+	mrr->rate2 = amn->amn_tx_rate2sp;
+	mrr->retries2 = amn->amn_tx_try2;
+	mrr->rate3 = amn->amn_tx_rate3sp;
+	mrr->retries3 = amn->amn_tx_try3;
 }
-EXPORT_SYMBOL(ath_rate_setupxtxdesc);
 
-void
+static void
 ath_rate_tx_complete(struct ath_softc *sc,
-	struct ath_node *an, const struct ath_desc *ds)
+	struct ath_node *an, const struct ath_buf *bf)
 {
 	struct amrr_node *amn = ATH_NODE_AMRR(an);
-	int sr = ds->ds_txstat.ts_shortretry;
-	int lr = ds->ds_txstat.ts_longretry;
+	const struct ath_tx_status *ts = &bf->bf_dsstatus.ds_txstat;
+	int sr = ts->ts_shortretry;
+	int lr = ts->ts_longretry;
 	int retry_count = sr + lr;
 
 	amn->amn_tx_try0_cnt++;
@@ -158,17 +157,15 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		amn->amn_tx_failure_cnt++;
 	}
 }
-EXPORT_SYMBOL(ath_rate_tx_complete);
 
-void
+static void
 ath_rate_newassoc(struct ath_softc *sc, struct ath_node *an, int isnew)
 {
 	if (isnew)
 		ath_rate_ctl_start(sc, &an->an_node);
 }
-EXPORT_SYMBOL(ath_rate_newassoc);
 
-static void 
+static void
 node_reset (struct amrr_node *amn)
 {
 	amn->amn_tx_try0_cnt = 0;
@@ -273,7 +270,7 @@ ath_rate_ctl_start(struct ath_softc *sc, struct ieee80211_node *ni)
 	int srate;
 
 	KASSERT(ni->ni_rates.rs_nrates > 0, ("no rates"));
-	if (vap->iv_fixed_rate == -1) {
+	if (vap->iv_fixed_rate == IEEE80211_FIXED_RATE_NONE) {
 		/*
 		 * No fixed rate is requested. For 11b start with
 		 * the highest negotiated rate; otherwise, for 11g
@@ -315,7 +312,7 @@ ath_rate_cb(void *arg, struct ieee80211_node *ni)
 /*
  * Reset the rate control state for each 802.11 state transition.
  */
-void
+static void
 ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state state)
 {
 	struct ieee80211com *ic = vap->iv_ic;
@@ -340,14 +337,14 @@ ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state state)
 	} else {
 		/*
 		 * When operating as a station the node table holds
-		 * the AP's that were discovered during scanning.
+		 * the APs that were discovered during scanning.
 		 * For any other operating mode we want to reset the
 		 * tx rate state of each node.
 		 */
 		ieee80211_iterate_nodes(&ic->ic_sta, ath_rate_cb, NULL);
 		ath_rate_update(sc, vap->iv_bss, 0);
 	}
-	if (vap->iv_fixed_rate == -1 && state == IEEE80211_S_RUN) {
+	if ((vap->iv_fixed_rate == IEEE80211_FIXED_RATE_NONE) && (state == IEEE80211_S_RUN)) {
 		int interval;
 		/*
 		 * Start the background rate control thread if we
@@ -359,7 +356,6 @@ ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state state)
 		mod_timer(&asc->timer, jiffies + ((HZ * interval) / 1000));
 	}
 }
-EXPORT_SYMBOL(ath_rate_newstate);
 
 /* 
  * Examine and potentially adjust the transmit rate.
@@ -378,7 +374,7 @@ ath_rate_ctl(void *arg, struct ieee80211_node *ni)
 #define is_min_rate(ni) (ni->ni_txrate == 0)
 
 	old_rate = ni->ni_txrate;
-  
+
   	DPRINTF (sc, "cnt0: %d cnt1: %d cnt2: %d cnt3: %d -- threshold: %d\n",
 		 amn->amn_tx_try0_cnt,
 		 amn->amn_tx_try1_cnt,
@@ -455,14 +451,17 @@ ath_ratectl(unsigned long data)
 	add_timer(&asc->timer);
 }
 
-struct ath_ratectrl *
+static struct ath_ratectrl *
 ath_rate_attach(struct ath_softc *sc)
 {
 	struct amrr_softc *asc;
 
+	_MOD_INC_USE(THIS_MODULE, return NULL);
 	asc = kmalloc(sizeof(struct amrr_softc), GFP_ATOMIC);
-	if (asc == NULL)
+	if (asc == NULL) {
+		_MOD_DEC_USE(THIS_MODULE);
 		return NULL;
+	}
 	asc->arc.arc_space = sizeof(struct amrr_node);
 	asc->arc.arc_vap_space = 0;
 	init_timer(&asc->timer);
@@ -471,39 +470,24 @@ ath_rate_attach(struct ath_softc *sc)
 
 	return &asc->arc;
 }
-EXPORT_SYMBOL(ath_rate_attach);
 
-void
+static void
 ath_rate_detach(struct ath_ratectrl *arc)
 {
 	struct amrr_softc *asc = (struct amrr_softc *) arc;
 
 	del_timer(&asc->timer);
 	kfree(asc);
+	_MOD_DEC_USE(THIS_MODULE);
 }
-EXPORT_SYMBOL(ath_rate_detach);
-
-#ifdef CONFIG_SYSCTL
-void
-ath_rate_dynamic_proc_register(struct ieee80211vap *vap)
-{		
-        /* Amrr rate module reports no statistics */
-}
-EXPORT_SYMBOL(ath_rate_dynamic_proc_register);
-#endif /* CONFIG_SYSCTL */
 
 static int minrateinterval = 500;	/* 500ms */
 static int maxint = 0x7fffffff;		/* 32-bit big */
 static int min_threshold = 1;
 
-#define	CTL_AUTO	-2	/* cannot be CTL_ANY or CTL_NONE */
-
 /*
  * Static (i.e. global) sysctls.
  */
-enum {
-	DEV_ATH		= 9,			/* XXX known by many */
-};
 
 static ctl_table ath_rate_static_sysctls[] = {
 	{ .ctl_name	= CTL_AUTO,
@@ -558,6 +542,19 @@ static ctl_table ath_root_table[] = {
 };
 static struct ctl_table_header *ath_sysctl_header;
 
+static struct ieee80211_rate_ops ath_rate_ops = {
+	.ratectl_id = IEEE80211_RATE_AMRR,
+	.node_init = ath_rate_node_init,
+	.node_cleanup = ath_rate_node_cleanup,
+	.findrate = ath_rate_findrate,
+	.get_mrr = ath_rate_get_mrr,
+	.tx_complete = ath_rate_tx_complete,
+	.newassoc = ath_rate_newassoc,
+	.newstate = ath_rate_newstate,
+	.attach = ath_rate_attach,
+	.detach = ath_rate_detach,
+};
+
 #include "release.h"
 static char *version = "0.1 (" RELEASE_VERSION ")";
 static char *dev_info = "ath_rate_amrr";
@@ -574,11 +571,14 @@ MODULE_LICENSE("Dual BSD/GPL");
 static int __init
 init_ath_rate_amrr(void)
 {
+	int ret;
 	printk(KERN_INFO "%s: %s\n", dev_info, version);
 
-#ifdef CONFIG_SYSCTL
-	ath_sysctl_header = register_sysctl_table(ath_root_table, 1);
-#endif
+	ret = ieee80211_rate_register(&ath_rate_ops);
+	if (ret)
+		return ret;
+
+	ath_sysctl_header = ATH_REGISTER_SYSCTL_TABLE(ath_root_table);
 	return (0);
 }
 module_init(init_ath_rate_amrr);
@@ -586,10 +586,9 @@ module_init(init_ath_rate_amrr);
 static void __exit
 exit_ath_rate_amrr(void)
 {
-#ifdef CONFIG_SYSCTL
 	if (ath_sysctl_header != NULL)
 		unregister_sysctl_table(ath_sysctl_header);
-#endif
+	ieee80211_rate_unregister(&ath_rate_ops);
 
 	printk(KERN_INFO "%s: unloaded\n", dev_info);
 }
