@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: sample.c 1721 2006-09-20 08:45:13Z mentor $
+ * $Id: sample.c 2874 2007-11-21 08:23:20Z mtaylor $
  */
 
 
@@ -60,9 +60,10 @@
 
 #include <net80211/if_media.h>
 #include <net80211/ieee80211_var.h>
+#include <net80211/ieee80211_rate.h>
 
-#include "if_athrate.h"
 #include "if_athvar.h"
+#include "if_ath_hal.h"
 #include "ah_desc.h"
 
 #include "sample.h"
@@ -70,14 +71,17 @@
 #define	SAMPLE_DEBUG
 #ifdef SAMPLE_DEBUG
 enum {
-	ATH_DEBUG_RATE		= 0x00000010	/* rate control */
+	ATH_DEBUG_RATE		= 0x00000010,	/* rate control */
+	ATH_DEBUG_ANY		= 0xffffffff
 };
-#define	DPRINTF(sc, _fmt, ...) do {				\
-	if (sc->sc_debug & ATH_DEBUG_RATE)			\
-		printf(_fmt, __VA_ARGS__);			\
+#define	DPRINTF(sc, m, fmt, ...) do {				\
+	if (sc->sc_debug & (m))					\
+		printf(fmt, __VA_ARGS__);			\
 } while (0)
 #else
-#define	DPRINTF(sc, _fmt, ...)
+#define	DPRINTF(sc, m, fmt, ...) do {				\
+	(void) sc;						\
+} while (0)
 #endif
 
 /*
@@ -122,26 +126,26 @@ static int ath_sample_rate = 10;		/* use x% of transmission time
 static void ath_rate_ctl_reset(struct ath_softc *, struct ieee80211_node *);
 
 
-static __inline int 
-size_to_bin(int size) 
+static __inline int
+size_to_bin(int size)
 {
-	int x = 0;
+	unsigned int x;
 	for (x = 0; x < NUM_PACKET_SIZE_BINS; x++)
 		if (size <= packet_size_bins[x])
 			return x;
-		
+
 	return NUM_PACKET_SIZE_BINS - 1;
 }
 
-static __inline int 
+static __inline int
 bin_to_size(int index) {
 	return packet_size_bins[index];
 }
 
-static __inline int 
+static __inline int
 rate_to_ndx(struct sample_node *sn, int rate)
 {
-	int x = 0;
+	unsigned int x;
 	for (x = 0; x < sn->num_rates; x++)
 		if (sn->rates[x].rate == rate)
 			return x;
@@ -152,30 +156,32 @@ rate_to_ndx(struct sample_node *sn, int rate)
  * Calculate the transmit duration of a frame.
  */
 static unsigned
-calc_usecs_unicast_packet(struct ath_softc *sc, int length, 
+calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 	int rix, int short_retries, int long_retries)
 {
 	const HAL_RATE_TABLE *rt = sc->sc_currates;
 	int rts, cts;
-	
-	unsigned t_slot = 20;
-	unsigned t_difs = 50; 
-	unsigned t_sifs = 10; 
+
+	unsigned t_slot;
+	unsigned t_difs;
+	unsigned t_sifs;
 	struct ieee80211com *ic = &sc->sc_ic;
-	int tt = 0;
-	int x = 0;
-	int cw = WIFI_CW_MIN;
-	int cix = rt->info[rix].controlRate;
+	unsigned int tt = 0;
+	unsigned int x;
+	unsigned int cw = WIFI_CW_MIN;
+	unsigned int cix = rt->info[rix].controlRate;
 	KASSERT(rt != NULL, ("no rate table, mode %u", sc->sc_curmode));
 
 	if (!rt->info[rix].rateKbps) {
-		printk(KERN_WARNING "rix %d (%d) bad ratekbps %d mode %u\n",
+		printk(KERN_WARNING "rix %u (%u) bad ratekbps %u mode %u\n",
 		       rix, rt->info[rix].dot11Rate,
 		       rt->info[rix].rateKbps,
 		       sc->sc_curmode);
 
 		return 0;
 	}
+
+	cix = rt->info[rix].controlRate;
 	/* 
 	 * XXX getting mac/phy level timings should be fixed for turbo
 	 * rates, and there is probably a way to get this from the
@@ -217,19 +223,19 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 		rts = 1;
 
 	if (rts || cts) {
-		int ctsrate = rt->info[cix].rateCode;
+		int ctsrate;
 		int ctsduration = 0;
 
 		if (!rt->info[cix].rateKbps) {
-			printk(KERN_WARNING "cix %d (%d) bad ratekbps %d mode %u\n",
+			printk(KERN_WARNING "cix %u (%u) bad ratekbps %u mode %u\n",
 			       cix, rt->info[cix].dot11Rate,
 			       rt->info[cix].rateKbps,
 			       sc->sc_curmode);
 			return 0;
 		}
-		
 
-		ctsrate |= rt->info[cix].shortPreamble;
+
+		ctsrate = rt->info[cix].rateCode | rt->info[cix].shortPreamble;
 		if (rts)		/* SIFS + CTS */
 			ctsduration += rt->info[cix].spAckDuration;
 
@@ -243,7 +249,7 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 	}
 	tt += t_difs;
 	tt += (long_retries+1)*(t_sifs + rt->info[rix].spAckDuration);
-	tt += (long_retries+1)*ath_hal_computetxtime(sc->sc_ah, rt, length, 
+	tt += (long_retries+1)*ath_hal_computetxtime(sc->sc_ah, rt, length,
 						rix, AH_TRUE);
 	for (x = 0; x <= short_retries + long_retries; x++) {
 		cw = MIN(WIFI_CW_MAX, (cw + 1) * 2);
@@ -252,21 +258,20 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 	return tt;
 }
 
-void
+static void
 ath_rate_node_init(struct ath_softc *sc, struct ath_node *an)
 {
 	/* NB: assumed to be zero'd by caller */
 	ath_rate_ctl_reset(sc, &an->an_node);
 }
-EXPORT_SYMBOL(ath_rate_node_init);
 
-void
+static void
 ath_rate_node_cleanup(struct ath_softc *sc, struct ath_node *an)
 {
 }
-EXPORT_SYMBOL(ath_rate_node_cleanup);
 
-void
+#if 0
+static void
 ath_rate_node_copy(struct ath_softc *sc,
 	struct ath_node *dst, const struct ath_node *src)
 {
@@ -274,26 +279,27 @@ ath_rate_node_copy(struct ath_softc *sc,
 	const struct sample_node *osrc = (const struct sample_node *)&src[1];
 	memcpy(odst, osrc, sizeof(struct sample_node));
 }
-EXPORT_SYMBOL(ath_rate_node_copy);
+#endif
 
 /*
  * returns the ndx with the lowest average_tx_time,
  * or -1 if all the average_tx_times are 0.
  */
-static __inline int best_rate_ndx(struct sample_node *sn, int size_bin, 
+static __inline int best_rate_ndx(struct sample_node *sn, int size_bin,
 	int require_acked_before)
 {
-	int x = 0;
-        int best_rate_ndx = 0;
-        int best_rate_tt = 0;
-        for (x = 0; x < sn->num_rates; x++) {
-		int tt = sn->stats[size_bin][x].average_tx_time;
-		if (tt <= 0 || (require_acked_before && 
+	unsigned int x;
+	unsigned int best_rate_tt = 0;
+	unsigned int best_rate_ndx = -1;
+
+	for (x = 0; x < sn->num_rates; x++) {
+		unsigned int tt = sn->stats[size_bin][x].average_tx_time;
+		if (tt <= 0 || (require_acked_before &&
 		    !sn->stats[size_bin][x].packets_acked))
 			continue;
 
 		/* 9 megabits never works better than 12 */
-		if (sn->rates[x].rate == 18) 
+		if (sn->rates[x].rate == 18)
 			continue;
 
 		/* don't use a bit-rate that has been failing */
@@ -304,37 +310,37 @@ static __inline int best_rate_ndx(struct sample_node *sn, int size_bin,
 			best_rate_tt = tt;
 			best_rate_ndx = x;
 		}
-        }
-        return (best_rate_tt) ? best_rate_ndx : -1;
+	}
+	return best_rate_ndx;
 }
 
 /*
  * pick a good "random" bit-rate to sample other than the current one
  */
-static __inline int 
-pick_sample_ndx(struct sample_node *sn, int size_bin) 
+static __inline int
+pick_sample_ndx(struct sample_node *sn, int size_bin)
 {
-	int x = 0;
-	int current_ndx = 0;
-	unsigned current_tt = 0;
-	
+	unsigned int x;
+	unsigned current_tt;
+	int current_ndx;
+
 	current_ndx = sn->current_rate[size_bin];
 	if (current_ndx < 0) {
 		/* no successes yet, send at the lowest bit-rate */
 		return 0;
 	}
-	
-	current_tt = sn->stats[size_bin][current_ndx].average_tx_time;
-	
-	for (x = 0; x < sn->num_rates; x++) {
-		int ndx = (sn->last_sample_ndx[size_bin]+1+x) % sn->num_rates;
 
-	        /* don't sample the current bit-rate */
-		if (ndx == current_ndx) 
+	current_tt = sn->stats[size_bin][current_ndx].average_tx_time;
+
+	for (x = 0; x < sn->num_rates; x++) {
+		unsigned int ndx = (sn->last_sample_ndx[size_bin] + 1 + x) % sn->num_rates;
+
+		/* don't sample the current bit-rate */
+		if (ndx == current_ndx)
 			continue;
 
 		/* this bit-rate is always worse than the current one */
-		if (sn->stats[size_bin][ndx].perfect_tx_time > current_tt) 
+		if (sn->stats[size_bin][ndx].perfect_tx_time > current_tt)
 			continue;
 
 		/* rarely sample bit-rates that fail a lot */
@@ -349,12 +355,12 @@ pick_sample_ndx(struct sample_node *sn, int size_bin)
 			continue;
 
 		/* 9 megabits never works better than 12 */
-		if (sn->rates[ndx].rate == 18) 
+		if (sn->rates[ndx].rate == 18)
 			continue;
 
 		/* if we're using 11 megabits, only sample up to 12 megabits
 		 */
-		if (sn->rates[current_ndx].rate == 22 && ndx > current_ndx + 1) 
+		if (sn->rates[current_ndx].rate == 22 && ndx > current_ndx + 1)
 			continue;
 
 		sn->last_sample_ndx[size_bin] = ndx;
@@ -363,20 +369,21 @@ pick_sample_ndx(struct sample_node *sn, int size_bin)
 	return current_ndx;
 }
 
-void
+static void
 ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 	int shortPreamble, size_t frameLen,
-	u_int8_t *rix, int *try0, u_int8_t *txrate)
+	u_int8_t *rix, unsigned int *try0, u_int8_t *txrate)
 {
 	struct sample_node *sn = ATH_NODE_SAMPLE(an);
 	struct sample_softc *ssc = ATH_SOFTC_SAMPLE(sc);
 	struct ieee80211com *ic = &sc->sc_ic;
-	int ndx, size_bin, mrr, best_ndx, change_rates;
+	unsigned int size_bin, mrr, change_rates;
+	int ndx, best_ndx;
 	unsigned average_tx_time;
 
 	if (sn->num_rates <= 0) {
 		printk(KERN_WARNING "%s: no rates for %s?\n",
-		       dev_info, 
+		       dev_info,
 		       ether_sprintf(an->an_node.ni_macaddr));
 		return;
 	}
@@ -389,13 +396,13 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 		average_tx_time = sn->stats[size_bin][best_ndx].average_tx_time;
 	else
 		average_tx_time = 0;
-	
+
 	if (sn->static_rate_ndx != -1) {
 		ndx = sn->static_rate_ndx;
 		*try0 = ATH_TXMAXTRY;
 	} else {
 		*try0 = mrr ? 2 : ATH_TXMAXTRY;
-		
+
 		if (sn->sample_tt[size_bin] < average_tx_time * (sn->packets_since_sample[size_bin] * ssc->ath_sample_rate / 100)) {
 			/*
 			 * we want to limit the time measuring the performance
@@ -431,7 +438,7 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 					 * pick the highest rate <= initial_rate/2 Mbps
 					 * that hasn't failed.
 					 */
-					if (sn->rates[ndx].rate <= initial_rate && 
+					if (sn->rates[ndx].rate <= initial_rate &&
 					    sn->stats[size_bin][ndx].successive_failures == 0)
 						break;
 				}
@@ -449,10 +456,10 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 			}
 
 			sn->packets_since_sample[size_bin]++;
-			
+
 			if (change_rates) {
 				if (best_ndx != sn->current_rate[size_bin]) {
-					DPRINTF(sc, "%s: %s size %d switch rate %d (%d/%d) -> %d (%d/%d) after %d packets mrr %d\n",
+					DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s size %u switch rate %u (%u/%u) -> %u (%u/%u) after %u packets mrr %u\n",
 						dev_info,
 						ether_sprintf(an->an_node.ni_macaddr),
 						packet_size_bins[size_bin],
@@ -472,8 +479,8 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 			ndx = sn->current_rate[size_bin];
 			sn->packets_since_switch[size_bin]++;
 			if (size_bin == 0) {
-	    			/* 
-	    			 * set the visible txrate for this node
+				/* 
+				 * set the visible txrate for this node
 			         * to the rate of small packets
 			         */
 				an->an_node.ni_txrate = ndx;
@@ -481,11 +488,11 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 		}
 	}
 
-	KASSERT(ndx >= 0 && ndx < sn->num_rates, 
-		("%s: bad ndx (%d/%d) for %s?\n",
-		 dev_info, ndx, sn->num_rates, 
+	KASSERT(ndx >= 0 && ndx < sn->num_rates,
+		("%s: bad ndx (%u/%u) for %s?\n",
+		 dev_info, ndx, sn->num_rates,
 		 ether_sprintf(an->an_node.ni_macaddr)));
-		
+
 
 	*rix = sn->rates[ndx].rix;
 	if (shortPreamble)
@@ -494,37 +501,35 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 		*txrate = sn->rates[ndx].rateCode;
 	sn->packets_sent[size_bin]++;
 }
-EXPORT_SYMBOL(ath_rate_findrate);
 
-void
-ath_rate_setupxtxdesc(struct ath_softc *sc, struct ath_node *an,
-	struct ath_desc *ds, int shortPreamble, size_t frame_size, u_int8_t rix)
+static void
+ath_rate_get_mrr(struct ath_softc *sc, struct ath_node *an, int shortPreamble,
+		 size_t frame_size, u_int8_t rix, struct ieee80211_mrr *mrr)
 {
 	struct sample_node *sn = ATH_NODE_SAMPLE(an);
-	int rateCode = -1;
-	int size_bin = 0;
-	int ndx = 0;
+	unsigned int size_bin;
+	int ndx;
 
 	size_bin = size_to_bin(frame_size);
 	ndx = sn->current_rate[size_bin]; /* retry at the current bit-rate */
-	
+
 	if (!sn->stats[size_bin][ndx].packets_acked)
 		ndx = 0;  /* use the lowest bit-rate */
 
 	if (shortPreamble)
-		rateCode = sn->rates[ndx].shortPreambleRateCode;
+		mrr->rate1 = sn->rates[ndx].shortPreambleRateCode;
 	else
-		rateCode = sn->rates[ndx].rateCode;
-	ath_hal_setupxtxdesc(sc->sc_ah, ds
-			     , rateCode, 3	        /* series 1 */
-			     , sn->rates[0].rateCode, 3	/* series 2 */
-			     , 0, 0	                /* series 3 */
-			     );
-}
-EXPORT_SYMBOL(ath_rate_setupxtxdesc);
+		mrr->rate1 = sn->rates[ndx].rateCode;
 
-static void 
-update_stats(struct ath_softc *sc, struct ath_node *an, 
+	mrr->retries1 = 3;
+	mrr->rate2 = sn->rates[0].rateCode;
+	mrr->retries2 = 3;
+	mrr->rate3 = 0;
+	mrr->retries3 = 0;
+}
+
+static void
+update_stats(struct ath_softc *sc, struct ath_node *an,
 	int frame_size,
 	int ndx0, int tries0,
 	int ndx1, int tries1,
@@ -535,14 +540,20 @@ update_stats(struct ath_softc *sc, struct ath_node *an,
 	const HAL_RATE_TABLE *rt = sc->sc_currates;
 	struct sample_node *sn = ATH_NODE_SAMPLE(an);
 	struct sample_softc *ssc = ATH_SOFTC_SAMPLE(sc);
-	int tt = 0;
-	int tries_so_far = 0;
-	int size_bin = 0;
-	int size = 0;
-	int rate = 0;
+	unsigned int tt = 0;
+	unsigned int tries_so_far = 0;
+	unsigned int size_bin;
+	unsigned int size;
+	unsigned int rate;
 
 	size_bin = size_to_bin(frame_size);
 	size = bin_to_size(size_bin);
+
+	if (!(0 <= ndx0 && ndx0 < sn->num_rates)) {
+		printk("%s: bogus ndx0 %d, max %u, mode %u\n",
+				__func__, ndx0, sn->num_rates, sc->sc_curmode);
+		return;
+	}
 	rate = sn->rates[ndx0].rate;
 
 	if (!rt->info[ndx0].rateKbps) {
@@ -552,48 +563,63 @@ update_stats(struct ath_softc *sc, struct ath_node *an,
 		 */
 		return;
 	}
-	tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx0].rix, 
-		short_tries-1, 
+	tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx0].rix,
+		short_tries,
 		MIN(tries0, tries) - 1);
 	tries_so_far += tries0;
-	if (tries1 && tries0 < tries) {
-		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx1].rix, 
-			short_tries - 1, 
+	if (tries1 && (tries0 < tries)) {
+		if (!(0 <= ndx1 && ndx1 < sn->num_rates)) {
+			printk("%s: bogus ndx1 %d, max %u, mode %u\n",
+					__func__, ndx1, sn->num_rates, sc->sc_curmode);
+			return;
+		}
+		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx1].rix,
+			short_tries,
 			MIN(tries1 + tries_so_far, tries) - tries_so_far - 1);
 	}
 	tries_so_far += tries1;
 
-	if (tries2 && tries0 + tries1 < tries) {
-		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx2].rix, 
-			short_tries - 1,
+	if (tries2 && ((tries0 + tries1) < tries)) {
+		if (!(0 <= ndx2 && ndx2 < sn->num_rates)) {
+			printk("%s: bogus ndx2 %d, max %u, mode %u\n",
+					__func__, ndx2, sn->num_rates, sc->sc_curmode);
+			return;
+		}
+		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx2].rix,
+			short_tries,
 			MIN(tries2 + tries_so_far, tries) - tries_so_far - 1);
 	}
 
 	tries_so_far += tries2;
 
-	if (tries3 && tries0 + tries1 + tries2 < tries) {
-		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx3].rix, 
-			short_tries - 1, 
+	if (tries3 && ((tries0 + tries1 + tries2) < tries)) {
+		if (!(0 <= ndx3 && ndx3 < sn->num_rates)) {
+			printk("%s: bogus ndx3 %d, max %u, mode %u\n",
+					__func__, ndx3, sn->num_rates, sc->sc_curmode);
+			return;
+		}
+		tt += calc_usecs_unicast_packet(sc, size, sn->rates[ndx3].rix,
+			short_tries,
 			MIN(tries3 + tries_so_far, tries) - tries_so_far - 1);
 	}
-	
+
 	if (sn->stats[size_bin][ndx0].total_packets < (100 / (100 - ssc->ath_smoothing_rate))) {
 		/* just average the first few packets */
-		int avg_tx = sn->stats[size_bin][ndx0].average_tx_time;
-		int packets = sn->stats[size_bin][ndx0].total_packets;
+		unsigned int avg_tx = sn->stats[size_bin][ndx0].average_tx_time;
+		unsigned int packets = sn->stats[size_bin][ndx0].total_packets;
 		sn->stats[size_bin][ndx0].average_tx_time =
 			(tt + (avg_tx * packets)) / (packets + 1);
 	} else {
 		/* use a ewma */
-		sn->stats[size_bin][ndx0].average_tx_time = 
-			((sn->stats[size_bin][ndx0].average_tx_time * ssc->ath_smoothing_rate) + 
+		sn->stats[size_bin][ndx0].average_tx_time =
+			((sn->stats[size_bin][ndx0].average_tx_time * ssc->ath_smoothing_rate) +
 			 (tt * (100 - ssc->ath_smoothing_rate))) / 100;
 	}
-	
+
 	if (status) {
-		int y;
+		unsigned int y;
 		sn->stats[size_bin][ndx0].successive_failures++;
-		for (y = size_bin+1; y < NUM_PACKET_SIZE_BINS; y++) {
+		for (y = size_bin + 1; y < NUM_PACKET_SIZE_BINS; y++) {
 			/* also say larger packets failed since we
 			 * assume if a small packet fails at a lower
 			 * bit-rate then a larger one will also.
@@ -611,11 +637,10 @@ update_stats(struct ath_softc *sc, struct ath_node *an,
 	sn->stats[size_bin][ndx0].last_tx = jiffies;
 	sn->stats[size_bin][ndx0].total_packets++;
 
-
 	if (ndx0 == sn->current_sample_ndx[size_bin]) {
-		DPRINTF(sc, "%s: %s size %d sample rate %d tries (%d/%d) tt %d avg_tt (%d/%d) status %d\n", 
-			dev_info, ether_sprintf(an->an_node.ni_macaddr), 
-			size, rate, short_tries, tries, tt, 
+		DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s size %u sample rate %u tries (%u/%u) tt %u avg_tt (%u/%u) status %u\n",
+			dev_info, ether_sprintf(an->an_node.ni_macaddr),
+			size, rate, short_tries, tries, tt,
 			sn->stats[size_bin][ndx0].average_tx_time,
 			sn->stats[size_bin][ndx0].perfect_tx_time,
 			status);
@@ -624,30 +649,30 @@ update_stats(struct ath_softc *sc, struct ath_node *an,
 	}
 }
 
-void
+static void
 ath_rate_tx_complete(struct ath_softc *sc,
-	struct ath_node *an, const struct ath_desc *ds)
+	struct ath_node *an, const struct ath_buf *bf)
 {
 	struct sample_node *sn = ATH_NODE_SAMPLE(an);
 	struct ieee80211com *ic = &sc->sc_ic;
-	const struct ar5212_desc *ads = (const struct ar5212_desc *)&ds->ds_ctl0;
-	int final_rate = 0;
-	int short_tries = 0;
-	int long_tries = 0;
-	int ndx = -1;
-	int frame_size = 0;
-	int mrr;
+	const struct ath_tx_status *ts = &bf->bf_dsstatus.ds_txstat;
+	const struct ath_desc *ds = &bf->bf_desc[0];
+	unsigned int final_rate;
+	unsigned int short_tries;
+	unsigned int long_tries;
+	unsigned int frame_size;
+	unsigned int mrr;
 
-	final_rate = sc->sc_hwmap[ds->ds_txstat.ts_rate &~ HAL_TXSTAT_ALTRATE].ieeerate;
-	short_tries = ds->ds_txstat.ts_shortretry + 1;
-	long_tries = ds->ds_txstat.ts_longretry + 1;
+	final_rate = sc->sc_hwmap[ts->ts_rate &~ HAL_TXSTAT_ALTRATE].ieeerate;
+	short_tries = ts->ts_shortretry + 1;
+	long_tries = ts->ts_longretry + 1;
 	frame_size = ds->ds_ctl0 & 0x0fff; /* low-order 12 bits of ds_ctl0 */
 
 	if (frame_size == 0)
 		frame_size = 1500;
 
 	if (sn->num_rates <= 0) {
-		DPRINTF(sc, "%s: %s %s no rates yet\n", dev_info, 
+		DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s %s no rates yet\n", dev_info,
 			ether_sprintf(an->an_node.ni_macaddr), __func__);
 		return;
 	}
@@ -655,121 +680,137 @@ ath_rate_tx_complete(struct ath_softc *sc,
 	mrr = sc->sc_mrretry && !(ic->ic_flags & IEEE80211_F_USEPROT) && ENABLE_MRR;
 
 
-	if (sc->sc_mrretry && ds->ds_txstat.ts_status) {
+	if (sc->sc_mrretry && ts->ts_status) {
 		/* this packet failed */
-		DPRINTF(sc, "%s: %s size %d rate/try %d/%d %d/%d %d/%d %d/%d status %s retries (%d/%d)\n", 
+		DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s size %u rate/try %u/%u %u/%u %u/%u %u/%u status %s retries (%u/%u)\n",
 			dev_info,
 			ether_sprintf(an->an_node.ni_macaddr),
 			bin_to_size(size_to_bin(frame_size)),
-			sc->sc_hwmap[ads->xmit_rate0].ieeerate, ads->xmit_tries0,
-			sc->sc_hwmap[ads->xmit_rate1].ieeerate, ads->xmit_tries1,
-			sc->sc_hwmap[ads->xmit_rate2].ieeerate, ads->xmit_tries2,
-			sc->sc_hwmap[ads->xmit_rate3].ieeerate, ads->xmit_tries3,
-			ds->ds_txstat.ts_status ? "FAIL" : "OK",
-			short_tries, 
-			long_tries);
+			sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate0)].ieeerate,
+				MS(ds->ds_ctl2, AR_XmitDataTries0),
+			sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate1)].ieeerate,
+				MS(ds->ds_ctl2, AR_XmitDataTries1),
+			sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate2)].ieeerate,
+				MS(ds->ds_ctl2, AR_XmitDataTries2),
+			sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate3)].ieeerate,
+				MS(ds->ds_ctl2, AR_XmitDataTries3),
+			ts->ts_status ? "FAIL" : "OK",
+			short_tries, long_tries);
 	}
 
+	mrr = sc->sc_mrretry && !(ic->ic_flags & IEEE80211_F_USEPROT) && ENABLE_MRR;
 
-	if (!mrr || !(ds->ds_txstat.ts_rate & HAL_TXSTAT_ALTRATE)) {
+
+	if (!mrr || !(ts->ts_rate & HAL_TXSTAT_ALTRATE)) {
 		/* only one rate was used */
-		ndx = rate_to_ndx(sn, final_rate);
-		if (ndx >= 0 && ndx < sn->num_rates) {
-			update_stats(sc, an, frame_size, 
+		int ndx = rate_to_ndx(sn, final_rate);
+		if ((ndx >= 0) && (ndx < sn->num_rates)) {
+			update_stats(sc, an, frame_size,
 				ndx, long_tries,
 				0, 0,
 				0, 0,
 				0, 0,
-				short_tries, long_tries, ds->ds_txstat.ts_status);
+				short_tries, long_tries, ts->ts_status);
 		}
 	} else {
-		int rate0, tries0, ndx0;
-		int rate1, tries1, ndx1;
-		int rate2, tries2, ndx2;
-		int rate3, tries3, ndx3;
-		int finalTSIdx = ads->final_ts_index;
+		unsigned int rate[4], tries[4];
+		int ndx[4];
+		int finalTSIdx = ts->ts_finaltsi;
 
 		/*
 		 * Process intermediate rates that failed.
 		 */
 
-		rate0 = sc->sc_hwmap[ads->xmit_rate0].ieeerate;
-		tries0 = ads->xmit_tries0;
-		ndx0 = rate_to_ndx(sn, rate0);
-		
-		rate1 = sc->sc_hwmap[ads->xmit_rate1].ieeerate;
-		tries1 = ads->xmit_tries1;
-		ndx1 = rate_to_ndx(sn, rate1);
-		
-		rate2 = sc->sc_hwmap[ads->xmit_rate2].ieeerate;
-		tries2 = ads->xmit_tries2;
-		ndx2 = rate_to_ndx(sn, rate2);
-		
-		rate3 = sc->sc_hwmap[ads->xmit_rate3].ieeerate;
-		tries3 = ads->xmit_tries3;
-		ndx3 = rate_to_ndx(sn, rate3);
-		
+		rate[0] = sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate0)].ieeerate;
+		tries[0] = MS(ds->ds_ctl2, AR_XmitDataTries0);
+		ndx[0] = rate_to_ndx(sn, rate[0]);
+
+		rate[1] = sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate1)].ieeerate;
+		tries[1] = MS(ds->ds_ctl2, AR_XmitDataTries1);
+		ndx[1] = rate_to_ndx(sn, rate[1]);
+
+		rate[2] = sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate2)].ieeerate;
+		tries[2] = MS(ds->ds_ctl2, AR_XmitDataTries2);
+		ndx[2] = rate_to_ndx(sn, rate[2]);
+
+		rate[3] = sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate3)].ieeerate;
+		tries[3] = MS(ds->ds_ctl2, AR_XmitDataTries3);
+		ndx[3] = rate_to_ndx(sn, rate[3]);
+
 #if 0
-		DPRINTF(sc, "%s: %s size %d finaltsidx %d tries %d status %d rate/try %d/%d %d/%d %d/%d %d/%d\n", 
+		DPRINTF(sc, "%s: %s size %u finaltsidx %u tries %u status %u rate/try %u/%u %u/%u %u/%u %u/%u\n",
 			dev_info, ether_sprintf(an->an_node.ni_macaddr),
 			bin_to_size(size_to_bin(frame_size)),
 			finalTSIdx,
-			long_tries, 
+			long_tries,
 			ds->ds_txstat.ts_status,
-			rate0, tries0,
-			rate1, tries1,
-			rate2, tries2,
-			rate3, tries3);
+			rate[0], tries[0],
+			rate[1], tries[1],
+			rate[2], tries[2],
+			rate[3], tries[3]);
 #endif
 
-		if (tries0)
-			update_stats(sc, an, frame_size, 
-				ndx0, tries0, 
-				ndx1, tries1, 
-				ndx2, tries2, 
-				ndx3, tries3, 
-				short_tries, ds->ds_txstat.ts_longretry + 1, 
-				long_tries > tries0);
-		
-		if (tries1 && finalTSIdx > 0)
-			update_stats(sc, an, frame_size, 
-				ndx1, tries1, 
-				ndx2, tries2, 
-				ndx3, tries3, 
-				0, 0, 
-				short_tries, ds->ds_txstat.ts_longretry + 1 - tries0, 
-				ds->ds_txstat.ts_status);
+		/* NB: series > 0 are not penalized for failure
+		 * based on the try counts under the assumption
+		 * that losses are often bursty and since we 
+		 * sample higher rates 1 try at a time doing so
+		 * may unfairly penalize them.
+		 */
+		if (tries[0] && ndx[0] >= 0) {
+			update_stats(sc, an, frame_size,
+				ndx[0], tries[0],
+				ndx[1], tries[1],
+				ndx[2], tries[2],
+				ndx[3], tries[3],
+				short_tries, long_tries,
+				long_tries > tries[0]);
+			long_tries -= tries[0];
 
-		if (tries2 && finalTSIdx > 1)
-			update_stats(sc, an, frame_size, 
-				ndx2, tries2, 
-				ndx3, tries3, 
-				0, 0,
-				0, 0,
-				short_tries, ds->ds_txstat.ts_longretry + 1 - tries0 - tries1, 
-				ds->ds_txstat.ts_status);
+		}
 
-		if (tries3 && finalTSIdx > 2)
-			update_stats(sc, an, frame_size, 
-				ndx3, tries3, 
+		if (tries[1] && ndx[1] >= 0 && finalTSIdx > 0) {
+			update_stats(sc, an, frame_size,
+				ndx[1], tries[1],
+				ndx[2], tries[2],
+				ndx[3], tries[3],
+				0, 0,
+				short_tries, long_tries,
+				ts->ts_status);
+			long_tries -= tries[1];
+		}
+
+		if (tries[2] && ndx[2] >= 0 && finalTSIdx > 1) {
+			update_stats(sc, an, frame_size,
+				ndx[2], tries[2],
+				ndx[3], tries[3],
+				0, 0,
+				0, 0,
+				short_tries, long_tries,
+				ts->ts_status);
+			long_tries -= tries[2];
+		}
+
+		if (tries[3] && ndx[3] >= 0 && finalTSIdx > 2) {
+			update_stats(sc, an, frame_size,
+				ndx[3], tries[3],
 				0, 0,
 				0, 0,
 				0, 0,
-				short_tries, ds->ds_txstat.ts_longretry + 1 - tries0 - tries1 - tries2, 
-				ds->ds_txstat.ts_status);
+				short_tries, long_tries,
+				ts->ts_status);
+		}
+
 	}
 }
-EXPORT_SYMBOL(ath_rate_tx_complete);
 
-void
+static void
 ath_rate_newassoc(struct ath_softc *sc, struct ath_node *an, int isnew)
 {
-	DPRINTF(sc, "%s: %s %s\n", dev_info,
-		ether_sprintf(an->an_node.ni_macaddr), __func__);
+	DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s %s isnew %d\n", dev_info,
+		ether_sprintf(an->an_node.ni_macaddr), __func__, isnew);
 	if (isnew)
 		ath_rate_ctl_reset(sc, &an->an_node);
 }
-EXPORT_SYMBOL(ath_rate_newassoc);
 
 /*
  * Initialize the tables for a node.
@@ -781,45 +822,44 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 	struct sample_node *sn = ATH_NODE_SAMPLE(an);
 	struct ieee80211vap *vap = ni->ni_vap;
 	const HAL_RATE_TABLE *rt = sc->sc_currates;
-	int x, y;
-	int srate;
+	unsigned int x, y;
+	unsigned int srate;
+
 	sn->num_rates = 0;
 
 	if (rt == NULL) {
 		printk(KERN_WARNING "no rates yet! mode %u\n", sc->sc_curmode);
 		return;
 	}
-        sn->static_rate_ndx = -1;
+	sn->static_rate_ndx = -1;
 
 	sn->num_rates = ni->ni_rates.rs_nrates;
-        for (x = 0; x < ni->ni_rates.rs_nrates; x++) {
+	for (x = 0; x < ni->ni_rates.rs_nrates; x++) {
 		sn->rates[x].rate = ni->ni_rates.rs_rates[x] & IEEE80211_RATE_VAL;
 		sn->rates[x].rix = sc->sc_rixmap[sn->rates[x].rate];
 		if (sn->rates[x].rix == 0xff) {
-			DPRINTF(sc, "%s: %s ignore bogus rix at %d\n",
+			DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s ignore bogus rix at %u\n",
 				dev_info, __func__, x);
 			continue;
 		}
 		sn->rates[x].rateCode = rt->info[sn->rates[x].rix].rateCode;
-		sn->rates[x].shortPreambleRateCode = 
-			rt->info[sn->rates[x].rix].rateCode | 
+		sn->rates[x].shortPreambleRateCode =
+			rt->info[sn->rates[x].rix].rateCode |
 			rt->info[sn->rates[x].rix].shortPreamble;
 	}
-	
+
 	ni->ni_txrate = 0;
 	sn->num_rates = ni->ni_rates.rs_nrates;
 
 	if (sn->num_rates <= 0) {
-		DPRINTF(sc, "%s: %s %s no rates (fixed %d) \n",
+		DPRINTF(sc, ATH_DEBUG_RATE,"%s: %s %s no rates (fixed %u) \n",
 			dev_info, __func__, ether_sprintf(ni->ni_macaddr),
 			vap->iv_fixed_rate);
 		/* there are no rates yet we're done */
 		return;
 	}
 
-	if (vap->iv_fixed_rate != -1) {
-		srate = sn->num_rates - 1;
-
+	if (vap->iv_fixed_rate != IEEE80211_FIXED_RATE_NONE) {
 		/*
 		 * A fixed rate is to be used; ic_fixed_rate is an
 		 * index into the supported rate set.  Convert this
@@ -828,37 +868,44 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 		 * rate set is checked when the station associates.
 		 */
 		/* NB: the rate set is assumed sorted */
-		for (; srate >= 0 && (ni->ni_rates.rs_rates[srate] & IEEE80211_RATE_VAL) != vap->iv_fixed_rate; srate--);
+		for (x = 0, srate = 0; x < sn->num_rates; x++)
+			if ((ni->ni_rates.rs_rates[x] & IEEE80211_RATE_VAL) == vap->iv_fixed_rate)
+				srate = x;
 
-		KASSERT(srate >= 0,
-			("fixed rate %d not in rate set", vap->iv_fixed_rate));
+		KASSERT(((ni->ni_rates.rs_rates[srate] & IEEE80211_RATE_VAL) == vap->iv_fixed_rate),
+			("fixed rate %u not in rate set", vap->iv_fixed_rate));
 
 		sn->static_rate_ndx = srate;
 		ni->ni_txrate = srate;
-		DPRINTF(sc, "%s: %s %s fixed rate %d%sMbps\n",
-			dev_info, __func__, ether_sprintf(ni->ni_macaddr), 
+		DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s %s fixed rate %u%sMbps\n",
+			dev_info, __func__, ether_sprintf(ni->ni_macaddr),
 			sn->rates[srate].rate / 2,
 			(sn->rates[srate].rate % 0x1) ? ".5" : " ");
 		return;
 	}
-	
+
 
 	for (y = 0; y < NUM_PACKET_SIZE_BINS; y++) {
-		int size = bin_to_size(y);
+		unsigned int size = bin_to_size(y);
 		int ndx = 0;
 		sn->packets_sent[y] = 0;
 		sn->current_sample_ndx[y] = -1;
 		sn->last_sample_ndx[y] = 0;
-		
+
 		for (x = 0; x < ni->ni_rates.rs_nrates; x++) {
+			if (sn->rates[x].rix == 0xff) {
+				DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s ignore bogus rix at %u\n",
+					dev_info, __func__, x);
+				continue;
+			}
 			sn->stats[y][x].successive_failures = 0;
 			sn->stats[y][x].tries = 0;
 			sn->stats[y][x].total_packets = 0;
 			sn->stats[y][x].packets_acked = 0;
 			sn->stats[y][x].last_tx = 0;
 
-			sn->stats[y][x].perfect_tx_time = 
-				calc_usecs_unicast_packet(sc, size, 
+			sn->stats[y][x].perfect_tx_time =
+				calc_usecs_unicast_packet(sc, size,
 					sn->rates[x].rix,
 					0, 0);
 			sn->stats[y][x].average_tx_time =
@@ -867,14 +914,14 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 		}
 
 		/* set the initial rate */
-		for (ndx = sn->num_rates-1; ndx > 0; ndx--)
+		for (ndx = sn->num_rates - 1; ndx > 0; ndx--)
 			if (sn->rates[ndx].rate <= 72)
 				break;
 		sn->current_rate[y] = ndx;
 	}
 
-	DPRINTF(sc, "%s: %s %s %d rates %d%sMbps (%dus)- %d%sMbps (%dus)\n",
-		dev_info, __func__, ether_sprintf(ni->ni_macaddr), 
+	DPRINTF(sc, ATH_DEBUG_RATE, "%s: %s %s %u rates %u%sMbps (%uus)- %u%sMbps (%uus)\n",
+		dev_info, __func__, ether_sprintf(ni->ni_macaddr),
 		sn->num_rates,
 		sn->rates[0].rate / 2, sn->rates[0].rate % 0x1 ? ".5" : "",
 		sn->stats[1][0].perfect_tx_time,
@@ -888,17 +935,17 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 static void
 ath_rate_cb(void *arg, struct ieee80211_node *ni)
 {
-        ath_rate_ctl_reset(ni->ni_ic->ic_dev->priv, ni);
+	ath_rate_ctl_reset(ni->ni_ic->ic_dev->priv, ni);
 }
 
 /*
  * Reset the rate control state for each 802.11 state transition.
  */
-void
+static void
 ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state newstate)
 {
 	struct ieee80211com *ic = vap->iv_ic;
-	
+
 	if (newstate == IEEE80211_S_RUN) {
 		if (ic->ic_opmode != IEEE80211_M_STA) {
 			/*
@@ -909,17 +956,19 @@ ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state newstate)
 		ath_rate_newassoc(ic->ic_dev->priv, ATH_NODE(vap->iv_bss), 1);
 	}
 }
-EXPORT_SYMBOL(ath_rate_newstate);
 
-struct ath_ratectrl *
+static struct ath_ratectrl *
 ath_rate_attach(struct ath_softc *sc)
 {
 	struct sample_softc *osc;
-	DPRINTF(sc, "%s: %s\n", dev_info, __func__);
-	
+	DPRINTF(sc, ATH_DEBUG_ANY, "%s: %s\n", dev_info, __func__);
+
+	_MOD_INC_USE(THIS_MODULE, return NULL);
 	osc = kmalloc(sizeof(struct sample_softc), GFP_ATOMIC);
-	if (osc == NULL)
+	if (osc == NULL) {
+		_MOD_DEC_USE(THIS_MODULE);
 		return NULL;
+	}
 	osc->arc.arc_space = sizeof(struct sample_node);
 	osc->arc.arc_vap_space = 0;
 
@@ -928,17 +977,15 @@ ath_rate_attach(struct ath_softc *sc)
 
 	return &osc->arc;
 }
-EXPORT_SYMBOL(ath_rate_attach);
 
-void
+static void
 ath_rate_detach(struct ath_ratectrl *arc)
 {
 	struct sample_softc *osc = (struct sample_softc *) arc;
 	kfree(osc);
+	_MOD_DEC_USE(THIS_MODULE);
 }
-EXPORT_SYMBOL(ath_rate_detach);
 
-#ifdef CONFIG_SYSCTL
 static int
 proc_read_nodes(struct ieee80211vap *vap, const int size, char *buf, int space)
 {
@@ -946,11 +993,12 @@ proc_read_nodes(struct ieee80211vap *vap, const int size, char *buf, int space)
 	struct ieee80211_node *ni;
 	struct ath_node *an;
 	struct sample_node *sn;
-	struct ieee80211_node_table *nt = 
+	struct ieee80211_node_table *nt =
 		(struct ieee80211_node_table *) &vap->iv_ic->ic_sta;
-	int x = 0;
-	int size_bin = 0;
+	unsigned int ndx;
+	unsigned int size_bin;
 
+	IEEE80211_NODE_TABLE_LOCK_IRQ(nt);
 	TAILQ_FOREACH(ni, &nt->nt_node, ni_list) {
 		/* Assume each node needs 500 bytes */
 		if (buf + space < p + 500)
@@ -958,65 +1006,67 @@ proc_read_nodes(struct ieee80211vap *vap, const int size, char *buf, int space)
 		an = ATH_NODE(ni);
 		sn = ATH_NODE_SAMPLE(an);
 		/* Skip ourself */
-		if (memcmp(vap->iv_myaddr, ni->ni_macaddr, 
+		if (memcmp(vap->iv_myaddr, ni->ni_macaddr,
 					IEEE80211_ADDR_LEN)==0) {
 			continue;
 		}
 
 		size_bin = size_to_bin(size);
 		p += sprintf(p, "%s\n", ether_sprintf(ni->ni_macaddr));
-		p += sprintf(p, 
+		p += sprintf(p,
 				"rate\ttt\tperfect\tfailed\tpkts\tavg_tries\tlast_tx\n");
-		for (x = 0; x < sn->num_rates; x++) {
-			int a = 1;
-			int t = 1;
+		for (ndx = 0; ndx < sn->num_rates; ndx++) {
+			unsigned int a = 1;
+			unsigned int t = 1;
 
 			p += sprintf(p, "%s", 
-					(x == sn->current_rate[size_bin]) ? "*" : " ");
+					(ndx == sn->current_rate[size_bin]) ? "*" : " ");
 
-			p += sprintf(p, "%3d%s",
-					sn->rates[x].rate/2,
-					(sn->rates[x].rate & 0x1) != 0 ? ".5" : "  ");
+			p += sprintf(p, "%3u%s",
+					sn->rates[ndx].rate / 2,
+					(sn->rates[ndx].rate & 0x1) ? ".5" : ".0");
 
-			p += sprintf(p, "\t%4d\t%4d\t%2d\t%3d",
-					sn->stats[size_bin][x].average_tx_time,
-					sn->stats[size_bin][x].perfect_tx_time,
-					sn->stats[size_bin][x].successive_failures,
-					sn->stats[size_bin][x].total_packets);
+			p += sprintf(p, "\t%4u\t%4u\t%2u\t%6u",
+					sn->stats[size_bin][ndx].average_tx_time,
+					sn->stats[size_bin][ndx].perfect_tx_time,
+					sn->stats[size_bin][ndx].successive_failures,
+					sn->stats[size_bin][ndx].total_packets);
 
-			if (sn->stats[size_bin][x].total_packets) {
-				a = sn->stats[size_bin][x].total_packets;
-				t = sn->stats[size_bin][x].tries;
-			}
-			p += sprintf(p, "\t%d.%02d\t", t/a, (t*100/a) % 100);
-			if (sn->stats[size_bin][x].last_tx) {
-				unsigned d = jiffies - 
-					sn->stats[size_bin][x].last_tx;
-				p += sprintf(p, "%d.%02d", d / HZ, d % HZ);
+			if (sn->stats[size_bin][ndx].total_packets) {
+				a = sn->stats[size_bin][ndx].total_packets;
+				t = sn->stats[size_bin][ndx].tries;
+  			}
+
+			p += sprintf(p, "\t%u.%02u\t\t", t / a, (t * 100 / a) % 100);
+			if (sn->stats[size_bin][ndx].last_tx) {
+  				unsigned int d = jiffies - 
+					sn->stats[size_bin][ndx].last_tx;
+				p += sprintf(p, "%u.%02u", d / HZ, d % HZ);
 			} else {
 				p += sprintf(p, "-");
 			}
+
 			p += sprintf(p, "\n");
 		}
-		printk("\n");
 	}
+	IEEE80211_NODE_TABLE_UNLOCK_IRQ(nt);
 
 	return (p - buf);
 }
 
-int
+static int
 proc_ratesample_open(struct inode *inode, struct file *file)
 {
-	struct proc_ieee80211_priv *pv = NULL;
+	struct proc_ieee80211_priv *pv;
 	struct proc_dir_entry *dp = PDE(inode);
 	struct ieee80211vap *vap = dp->data;
-	int size = 0;
+	unsigned long size;
 
 	if (!(file->private_data = kmalloc(sizeof(struct proc_ieee80211_priv),
 					GFP_KERNEL)))
 		return -ENOMEM;
 
-	/* intially allocate both read and write buffers */
+	/* initially allocate both read and write buffers */
 	pv = (struct proc_ieee80211_priv *) file->private_data;
 	memset(pv, 0, sizeof(struct proc_ieee80211_priv));
 	pv->rbuf = vmalloc(MAX_PROC_IEEE80211_SIZE);
@@ -1036,7 +1086,7 @@ proc_ratesample_open(struct inode *inode, struct file *file)
 	pv->max_rlen = MAX_PROC_IEEE80211_SIZE;
 
 	/* Determine what size packets to get stats for based on proc filename */
-	size = (int)simple_strtol(dp->name + 10, NULL, 0);
+	size = simple_strtoul(dp->name + 10, NULL, 0);
 
 	/* now read the data into the buffer */
 	pv->rlen = proc_read_nodes(vap, size, pv->rbuf, MAX_PROC_IEEE80211_SIZE);
@@ -1050,16 +1100,28 @@ static struct file_operations proc_ratesample_ops = {
 	.release = NULL,
 };
 
-void
+static void
 ath_rate_dynamic_proc_register(struct ieee80211vap *vap)
-{		
+{
 	/* Create proc entries for the rate control algorithm */
 	ieee80211_proc_vcreate(vap, &proc_ratesample_ops, "ratestats_250");
 	ieee80211_proc_vcreate(vap, &proc_ratesample_ops, "ratestats_1600");
-	ieee80211_proc_vcreate(vap, &proc_ratesample_ops, "ratestats_3000");	
+	ieee80211_proc_vcreate(vap, &proc_ratesample_ops, "ratestats_3000");
 }
-EXPORT_SYMBOL(ath_rate_dynamic_proc_register);
-#endif /* CONFIG_SYSCTL */
+
+static struct ieee80211_rate_ops ath_rate_ops = {
+	.ratectl_id = IEEE80211_RATE_SAMPLE,
+	.node_init = ath_rate_node_init,
+	.node_cleanup = ath_rate_node_cleanup,
+	.findrate = ath_rate_findrate,
+	.get_mrr = ath_rate_get_mrr,
+	.tx_complete = ath_rate_tx_complete,
+	.newassoc = ath_rate_newassoc,
+	.newstate = ath_rate_newstate,
+	.attach = ath_rate_attach,
+	.detach = ath_rate_detach,
+	.dynamic_proc_register = ath_rate_dynamic_proc_register,
+};
 
 MODULE_AUTHOR("John Bicket");
 MODULE_DESCRIPTION("SampleRate bit-rate selection algorithm for Atheros devices");
@@ -1074,13 +1136,14 @@ static int __init
 init_ath_rate_sample(void)
 {
 	printk(KERN_INFO "%s: %s\n", dev_info, version);
-	return (0);
+	return ieee80211_rate_register(&ath_rate_ops);
 }
 module_init(init_ath_rate_sample);
 
 static void __exit
 exit_ath_rate_sample(void)
 {
+	ieee80211_rate_unregister(&ath_rate_ops);
 	printk(KERN_INFO "%s: unloaded\n", dev_info);
 }
 module_exit(exit_ath_rate_sample);
