@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211.h 1762 2006-10-23 09:11:21Z kelmo $
+ * $Id: ieee80211.h 2885 2007-11-21 10:24:35Z mtaylor $
  */
 #ifndef _NET80211_IEEE80211_H_
 #define _NET80211_IEEE80211_H_
@@ -51,7 +51,7 @@ struct ieee80211_plcp_hdr {
 	u_int16_t i_crc;
 } __packed;
 
-#define IEEE80211_PLCP_SFD      0xF3A0 
+#define IEEE80211_PLCP_SFD      0xF3A0
 #define IEEE80211_PLCP_SERVICE  0x00
 
 /*
@@ -59,7 +59,7 @@ struct ieee80211_plcp_hdr {
  */
 struct ieee80211_frame {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr3[IEEE80211_ADDR_LEN];
@@ -70,7 +70,7 @@ struct ieee80211_frame {
 
 struct ieee80211_qosframe {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr3[IEEE80211_ADDR_LEN];
@@ -86,7 +86,7 @@ struct ieee80211_qoscntl {
 
 struct ieee80211_frame_addr4 {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr3[IEEE80211_ADDR_LEN];
@@ -97,7 +97,7 @@ struct ieee80211_frame_addr4 {
 
 struct ieee80211_qosframe_addr4 {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr3[IEEE80211_ADDR_LEN];
@@ -108,7 +108,7 @@ struct ieee80211_qosframe_addr4 {
 
 struct ieee80211_ctlframe_addr2 {
 	u_int8_t i_fc[2];
-	u_int8_t i_aidordur[2]; /* AID or duration */
+	__le16 i_aidordur; /* AID or duration */
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 } __packed;
@@ -192,164 +192,164 @@ struct ieee80211_ctlframe_addr2 {
  * XXX belongs elsewhere
  */
 enum CountryCode {
-    CTRY_ALBANIA              = 8,       /* Albania */
-    CTRY_ALGERIA              = 12,      /* Algeria */
-    CTRY_ARGENTINA            = 32,      /* Argentina */
-    CTRY_ARMENIA              = 51,      /* Armenia */
-    CTRY_AUSTRALIA            = 36,      /* Australia */
-    CTRY_AUSTRIA              = 40,      /* Austria */
-    CTRY_AZERBAIJAN           = 31,      /* Azerbaijan */
-    CTRY_BAHRAIN              = 48,      /* Bahrain */
-    CTRY_BELARUS              = 112,     /* Belarus */
-    CTRY_BELGIUM              = 56,      /* Belgium */
-    CTRY_BELIZE               = 84,      /* Belize */
-    CTRY_BOLIVIA              = 68,      /* Bolivia */
-    CTRY_BRAZIL               = 76,      /* Brazil */
-    CTRY_BRUNEI_DARUSSALAM    = 96,      /* Brunei Darussalam */
-    CTRY_BULGARIA             = 100,     /* Bulgaria */
-    CTRY_CANADA               = 124,     /* Canada */
-    CTRY_CHILE                = 152,     /* Chile */
-    CTRY_CHINA                = 156,     /* People's Republic of China */
-    CTRY_COLOMBIA             = 170,     /* Colombia */
-    CTRY_COSTA_RICA           = 188,     /* Costa Rica */
-    CTRY_CROATIA              = 191,     /* Croatia */
-    CTRY_CYPRUS               = 196,
-    CTRY_CZECH                = 203,     /* Czech Republic */
-    CTRY_DENMARK              = 208,     /* Denmark */
-    CTRY_DOMINICAN_REPUBLIC   = 214,     /* Dominican Republic */
-    CTRY_ECUADOR              = 218,     /* Ecuador */
-    CTRY_EGYPT                = 818,     /* Egypt */
-    CTRY_EL_SALVADOR          = 222,     /* El Salvador */
-    CTRY_ESTONIA              = 233,     /* Estonia */
-    CTRY_FAEROE_ISLANDS       = 234,     /* Faeroe Islands */
-    CTRY_FINLAND              = 246,     /* Finland */
-    CTRY_FRANCE               = 250,     /* France */
-    CTRY_FRANCE2              = 255,     /* France2 */
-    CTRY_GEORGIA              = 268,     /* Georgia */
-    CTRY_GERMANY              = 276,     /* Germany */
-    CTRY_GREECE               = 300,     /* Greece */
-    CTRY_GUATEMALA            = 320,     /* Guatemala */
-    CTRY_HONDURAS             = 340,     /* Honduras */
-    CTRY_HONG_KONG            = 344,     /* Hong Kong S.A.R., P.R.C. */
-    CTRY_HUNGARY              = 348,     /* Hungary */
-    CTRY_ICELAND              = 352,     /* Iceland */
-    CTRY_INDIA                = 356,     /* India */
-    CTRY_INDONESIA            = 360,     /* Indonesia */
-    CTRY_IRAN                 = 364,     /* Iran */
-    CTRY_IRAQ                 = 368,     /* Iraq */
-    CTRY_IRELAND              = 372,     /* Ireland */
-    CTRY_ISRAEL               = 376,     /* Israel */
-    CTRY_ITALY                = 380,     /* Italy */
-    CTRY_JAMAICA              = 388,     /* Jamaica */
-    CTRY_JAPAN                = 392,     /* Japan */
-    CTRY_JAPAN1               = 393,     /* Japan (JP1) */
-    CTRY_JAPAN2               = 394,     /* Japan (JP0) */
-    CTRY_JAPAN3               = 395,     /* Japan (JP1-1) */
-    CTRY_JAPAN4               = 396,     /* Japan (JE1) */
-    CTRY_JAPAN5               = 397,     /* Japan (JE2) */
-    CTRY_JAPAN6               = 399,	 /* Japan (JP6) */
-    CTRY_JAPAN7               = 900,	 /* Japan */
-    CTRY_JAPAN8               = 901,	 /* Japan */
-    CTRY_JAPAN9               = 902,	 /* Japan */
-    CTRY_JAPAN10	      = 903,	 /* Japan */
-    CTRY_JAPAN11	      = 904,	 /* Japan */
-    CTRY_JAPAN12	      = 905,	 /* Japan */
-    CTRY_JAPAN13	      = 906,	 /* Japan */
-    CTRY_JAPAN14	      = 907,	 /* Japan */
-    CTRY_JAPAN15	      = 908,	 /* Japan */
-    CTRY_JAPAN16	      = 909,	 /* Japan */
-    CTRY_JAPAN17	      = 910,	 /* Japan */
-    CTRY_JAPAN18	      = 911,	 /* Japan */
-    CTRY_JAPAN19	      = 912,	 /* Japan */
-    CTRY_JAPAN20	      = 913,	 /* Japan */
-    CTRY_JAPAN21	      = 914,	 /* Japan */
-    CTRY_JAPAN22	      = 915,	 /* Japan */
-    CTRY_JAPAN23	      = 916,	 /* Japan */
-    CTRY_JAPAN24	      = 917,	 /* Japan */
-    CTRY_JAPAN25	      = 918,	 /* Japan */
-    CTRY_JAPAN26	      = 919,	 /* Japan */
-    CTRY_JAPAN27	      = 920,	 /* Japan */
-    CTRY_JAPAN28	      = 921,	 /* Japan */
-    CTRY_JAPAN29	      = 922,	 /* Japan */
-    CTRY_JAPAN30	      = 923,	 /* Japan */
-    CTRY_JAPAN31	      = 924,	 /* Japan */
-    CTRY_JAPAN32	      = 925,	 /* Japan */
-    CTRY_JAPAN33	      = 926,	 /* Japan */
-    CTRY_JAPAN34	      = 927,	 /* Japan */
-    CTRY_JAPAN35	      = 928,	 /* Japan */
-    CTRY_JAPAN36	      = 929,	 /* Japan */
-    CTRY_JAPAN37	      = 930,	 /* Japan */
-    CTRY_JAPAN38	      = 931,	 /* Japan */
-    CTRY_JAPAN39	      = 932,	 /* Japan */
-    CTRY_JAPAN40	      = 933,	 /* Japan */
-    CTRY_JAPAN41	      = 934,	 /* Japan */
-    CTRY_JAPAN42	      = 935,	 /* Japan */
-    CTRY_JAPAN43	      = 936,	 /* Japan */
-    CTRY_JAPAN44	      = 937,	 /* Japan */
-    CTRY_JAPAN45	      = 938,	 /* Japan */
-    CTRY_JAPAN46	      = 939,	 /* Japan */
-    CTRY_JAPAN47	      = 940,	 /* Japan */
-    CTRY_JAPAN48	      = 941,	 /* Japan */
-    CTRY_JORDAN               = 400,     /* Jordan */
-    CTRY_KAZAKHSTAN           = 398,     /* Kazakhstan */
-    CTRY_KENYA                = 404,     /* Kenya */
-    CTRY_KOREA_NORTH          = 408,     /* North Korea */
-    CTRY_KOREA_ROC            = 410,     /* South Korea */
-    CTRY_KOREA_ROC2           = 411,     /* South Korea */
-    CTRY_KUWAIT               = 414,     /* Kuwait */
-    CTRY_LATVIA               = 428,     /* Latvia */
-    CTRY_LEBANON              = 422,     /* Lebanon */
-    CTRY_LIBYA                = 434,     /* Libya */
-    CTRY_LIECHTENSTEIN        = 438,     /* Liechtenstein */
-    CTRY_LITHUANIA            = 440,     /* Lithuania */
-    CTRY_LUXEMBOURG           = 442,     /* Luxembourg */
-    CTRY_MACAU                = 446,     /* Macau */
-    CTRY_MACEDONIA            = 807,     /* the Former Yugoslav Republic of Macedonia */
-    CTRY_MALAYSIA             = 458,     /* Malaysia */
-    CTRY_MEXICO               = 484,     /* Mexico */
-    CTRY_MONACO               = 492,     /* Principality of Monaco */
-    CTRY_MOROCCO              = 504,     /* Morocco */
-    CTRY_NETHERLANDS          = 528,     /* Netherlands */
-    CTRY_NEW_ZEALAND          = 554,     /* New Zealand */
-    CTRY_NICARAGUA            = 558,     /* Nicaragua */
-    CTRY_NORWAY               = 578,     /* Norway */
-    CTRY_OMAN                 = 512,     /* Oman */
-    CTRY_PAKISTAN             = 586,     /* Islamic Republic of Pakistan */
-    CTRY_PANAMA               = 591,     /* Panama */
-    CTRY_PARAGUAY             = 600,     /* Paraguay */
-    CTRY_PERU                 = 604,     /* Peru */
-    CTRY_PHILIPPINES          = 608,     /* Republic of the Philippines */
-    CTRY_POLAND               = 616,     /* Poland */
-    CTRY_PORTUGAL             = 620,     /* Portugal */
-    CTRY_PUERTO_RICO          = 630,     /* Puerto Rico */
-    CTRY_QATAR                = 634,     /* Qatar */
-    CTRY_ROMANIA              = 642,     /* Romania */
-    CTRY_RUSSIA               = 643,     /* Russia */
-    CTRY_SAUDI_ARABIA         = 682,     /* Saudi Arabia */
-    CTRY_SINGAPORE            = 702,     /* Singapore */
-    CTRY_SLOVAKIA             = 703,     /* Slovak Republic */
-    CTRY_SLOVENIA             = 705,     /* Slovenia */
-    CTRY_SOUTH_AFRICA         = 710,     /* South Africa */
-    CTRY_SPAIN                = 724,     /* Spain */
-    CTRY_SWEDEN               = 752,     /* Sweden */
-    CTRY_SWITZERLAND          = 756,     /* Switzerland */
-    CTRY_SYRIA                = 760,     /* Syria */
-    CTRY_TAIWAN               = 158,     /* Taiwan */
-    CTRY_THAILAND             = 764,     /* Thailand */
-    CTRY_TRINIDAD_Y_TOBAGO    = 780,     /* Trinidad y Tobago */
-    CTRY_TUNISIA              = 788,     /* Tunisia */
-    CTRY_TURKEY               = 792,     /* Turkey */
-    CTRY_UAE                  = 784,     /* U.A.E. */
-    CTRY_UKRAINE              = 804,     /* Ukraine */
-    CTRY_UNITED_KINGDOM       = 826,     /* United Kingdom */
-    CTRY_UNITED_STATES        = 840,     /* United States */
-    CTRY_UNITED_STATES_FCC49  = 842,     /* United States (Public Safety)*/
-    CTRY_URUGUAY              = 858,     /* Uruguay */
-    CTRY_UZBEKISTAN           = 860,     /* Uzbekistan */
-    CTRY_VENEZUELA            = 862,     /* Venezuela */
-    CTRY_VIET_NAM             = 704,     /* Viet Nam */
-    CTRY_YEMEN                = 887,     /* Yemen */
-    CTRY_ZIMBABWE             = 716      /* Zimbabwe */
+	CTRY_ALBANIA              = 8,       /* Albania */
+	CTRY_ALGERIA              = 12,      /* Algeria */
+	CTRY_ARGENTINA            = 32,      /* Argentina */
+	CTRY_ARMENIA              = 51,      /* Armenia */
+	CTRY_AUSTRALIA            = 36,      /* Australia */
+	CTRY_AUSTRIA              = 40,      /* Austria */
+	CTRY_AZERBAIJAN           = 31,      /* Azerbaijan */
+	CTRY_BAHRAIN              = 48,      /* Bahrain */
+	CTRY_BELARUS              = 112,     /* Belarus */
+	CTRY_BELGIUM              = 56,      /* Belgium */
+	CTRY_BELIZE               = 84,      /* Belize */
+	CTRY_BOLIVIA              = 68,      /* Bolivia */
+	CTRY_BRAZIL               = 76,      /* Brazil */
+	CTRY_BRUNEI_DARUSSALAM    = 96,      /* Brunei Darussalam */
+	CTRY_BULGARIA             = 100,     /* Bulgaria */
+	CTRY_CANADA               = 124,     /* Canada */
+	CTRY_CHILE                = 152,     /* Chile */
+	CTRY_CHINA                = 156,     /* People's Republic of China */
+	CTRY_COLOMBIA             = 170,     /* Colombia */
+	CTRY_COSTA_RICA           = 188,     /* Costa Rica */
+	CTRY_CROATIA              = 191,     /* Croatia */
+	CTRY_CYPRUS               = 196,
+	CTRY_CZECH                = 203,     /* Czech Republic */
+	CTRY_DENMARK              = 208,     /* Denmark */
+	CTRY_DOMINICAN_REPUBLIC   = 214,     /* Dominican Republic */
+	CTRY_ECUADOR              = 218,     /* Ecuador */
+	CTRY_EGYPT                = 818,     /* Egypt */
+	CTRY_EL_SALVADOR          = 222,     /* El Salvador */
+	CTRY_ESTONIA              = 233,     /* Estonia */
+	CTRY_FAEROE_ISLANDS       = 234,     /* Faeroe Islands */
+	CTRY_FINLAND              = 246,     /* Finland */
+	CTRY_FRANCE               = 250,     /* France */
+	CTRY_FRANCE2              = 255,     /* France2 */
+	CTRY_GEORGIA              = 268,     /* Georgia */
+	CTRY_GERMANY              = 276,     /* Germany */
+	CTRY_GREECE               = 300,     /* Greece */
+	CTRY_GUATEMALA            = 320,     /* Guatemala */
+	CTRY_HONDURAS             = 340,     /* Honduras */
+	CTRY_HONG_KONG            = 344,     /* Hong Kong S.A.R., P.R.C. */
+	CTRY_HUNGARY              = 348,     /* Hungary */
+	CTRY_ICELAND              = 352,     /* Iceland */
+	CTRY_INDIA                = 356,     /* India */
+	CTRY_INDONESIA            = 360,     /* Indonesia */
+	CTRY_IRAN                 = 364,     /* Iran */
+	CTRY_IRAQ                 = 368,     /* Iraq */
+	CTRY_IRELAND              = 372,     /* Ireland */
+	CTRY_ISRAEL               = 376,     /* Israel */
+	CTRY_ITALY                = 380,     /* Italy */
+	CTRY_JAMAICA              = 388,     /* Jamaica */
+	CTRY_JAPAN                = 392,     /* Japan */
+	CTRY_JAPAN1               = 393,     /* Japan (JP1) */
+	CTRY_JAPAN2               = 394,     /* Japan (JP0) */
+	CTRY_JAPAN3               = 395,     /* Japan (JP1-1) */
+	CTRY_JAPAN4               = 396,     /* Japan (JE1) */
+	CTRY_JAPAN5               = 397,     /* Japan (JE2) */
+	CTRY_JAPAN6               = 399,	 /* Japan (JP6) */
+	CTRY_JAPAN7               = 900,	 /* Japan */
+	CTRY_JAPAN8               = 901,	 /* Japan */
+	CTRY_JAPAN9               = 902,	 /* Japan */
+	CTRY_JAPAN10	      = 903,	 /* Japan */
+	CTRY_JAPAN11	      = 904,	 /* Japan */
+	CTRY_JAPAN12	      = 905,	 /* Japan */
+	CTRY_JAPAN13	      = 906,	 /* Japan */
+	CTRY_JAPAN14	      = 907,	 /* Japan */
+	CTRY_JAPAN15	      = 908,	 /* Japan */
+	CTRY_JAPAN16	      = 909,	 /* Japan */
+	CTRY_JAPAN17	      = 910,	 /* Japan */
+	CTRY_JAPAN18	      = 911,	 /* Japan */
+	CTRY_JAPAN19	      = 912,	 /* Japan */
+	CTRY_JAPAN20	      = 913,	 /* Japan */
+	CTRY_JAPAN21	      = 914,	 /* Japan */
+	CTRY_JAPAN22	      = 915,	 /* Japan */
+	CTRY_JAPAN23	      = 916,	 /* Japan */
+	CTRY_JAPAN24	      = 917,	 /* Japan */
+	CTRY_JAPAN25	      = 918,	 /* Japan */
+	CTRY_JAPAN26	      = 919,	 /* Japan */
+	CTRY_JAPAN27	      = 920,	 /* Japan */
+	CTRY_JAPAN28	      = 921,	 /* Japan */
+	CTRY_JAPAN29	      = 922,	 /* Japan */
+	CTRY_JAPAN30	      = 923,	 /* Japan */
+	CTRY_JAPAN31	      = 924,	 /* Japan */
+	CTRY_JAPAN32	      = 925,	 /* Japan */
+	CTRY_JAPAN33	      = 926,	 /* Japan */
+	CTRY_JAPAN34	      = 927,	 /* Japan */
+	CTRY_JAPAN35	      = 928,	 /* Japan */
+	CTRY_JAPAN36	      = 929,	 /* Japan */
+	CTRY_JAPAN37	      = 930,	 /* Japan */
+	CTRY_JAPAN38	      = 931,	 /* Japan */
+	CTRY_JAPAN39	      = 932,	 /* Japan */
+	CTRY_JAPAN40	      = 933,	 /* Japan */
+	CTRY_JAPAN41	      = 934,	 /* Japan */
+	CTRY_JAPAN42	      = 935,	 /* Japan */
+	CTRY_JAPAN43	      = 936,	 /* Japan */
+	CTRY_JAPAN44	      = 937,	 /* Japan */
+	CTRY_JAPAN45	      = 938,	 /* Japan */
+	CTRY_JAPAN46	      = 939,	 /* Japan */
+	CTRY_JAPAN47	      = 940,	 /* Japan */
+	CTRY_JAPAN48	      = 941,	 /* Japan */
+	CTRY_JORDAN               = 400,     /* Jordan */
+	CTRY_KAZAKHSTAN           = 398,     /* Kazakhstan */
+	CTRY_KENYA                = 404,     /* Kenya */
+	CTRY_KOREA_NORTH          = 408,     /* North Korea */
+	CTRY_KOREA_ROC            = 410,     /* South Korea */
+	CTRY_KOREA_ROC2           = 411,     /* South Korea */
+	CTRY_KUWAIT               = 414,     /* Kuwait */
+	CTRY_LATVIA               = 428,     /* Latvia */
+	CTRY_LEBANON              = 422,     /* Lebanon */
+	CTRY_LIBYA                = 434,     /* Libya */
+	CTRY_LIECHTENSTEIN        = 438,     /* Liechtenstein */
+	CTRY_LITHUANIA            = 440,     /* Lithuania */
+	CTRY_LUXEMBOURG           = 442,     /* Luxembourg */
+	CTRY_MACAU                = 446,     /* Macau */
+	CTRY_MACEDONIA            = 807,     /* the Former Yugoslav Republic of Macedonia */
+	CTRY_MALAYSIA             = 458,     /* Malaysia */
+	CTRY_MEXICO               = 484,     /* Mexico */
+	CTRY_MONACO               = 492,     /* Principality of Monaco */
+	CTRY_MOROCCO              = 504,     /* Morocco */
+	CTRY_NETHERLANDS          = 528,     /* Netherlands */
+	CTRY_NEW_ZEALAND          = 554,     /* New Zealand */
+	CTRY_NICARAGUA            = 558,     /* Nicaragua */
+	CTRY_NORWAY               = 578,     /* Norway */
+	CTRY_OMAN                 = 512,     /* Oman */
+	CTRY_PAKISTAN             = 586,     /* Islamic Republic of Pakistan */
+	CTRY_PANAMA               = 591,     /* Panama */
+	CTRY_PARAGUAY             = 600,     /* Paraguay */
+	CTRY_PERU                 = 604,     /* Peru */
+	CTRY_PHILIPPINES          = 608,     /* Republic of the Philippines */
+	CTRY_POLAND               = 616,     /* Poland */
+	CTRY_PORTUGAL             = 620,     /* Portugal */
+	CTRY_PUERTO_RICO          = 630,     /* Puerto Rico */
+	CTRY_QATAR                = 634,     /* Qatar */
+	CTRY_ROMANIA              = 642,     /* Romania */
+	CTRY_RUSSIA               = 643,     /* Russia */
+	CTRY_SAUDI_ARABIA         = 682,     /* Saudi Arabia */
+	CTRY_SINGAPORE            = 702,     /* Singapore */
+	CTRY_SLOVAKIA             = 703,     /* Slovak Republic */
+	CTRY_SLOVENIA             = 705,     /* Slovenia */
+	CTRY_SOUTH_AFRICA         = 710,     /* South Africa */
+	CTRY_SPAIN                = 724,     /* Spain */
+	CTRY_SWEDEN               = 752,     /* Sweden */
+	CTRY_SWITZERLAND          = 756,     /* Switzerland */
+	CTRY_SYRIA                = 760,     /* Syria */
+	CTRY_TAIWAN               = 158,     /* Taiwan */
+	CTRY_THAILAND             = 764,     /* Thailand */
+	CTRY_TRINIDAD_Y_TOBAGO    = 780,     /* Trinidad y Tobago */
+	CTRY_TUNISIA              = 788,     /* Tunisia */
+	CTRY_TURKEY               = 792,     /* Turkey */
+	CTRY_UAE                  = 784,     /* U.A.E. */
+	CTRY_UKRAINE              = 804,     /* Ukraine */
+	CTRY_UNITED_KINGDOM       = 826,     /* United Kingdom */
+	CTRY_UNITED_STATES        = 840,     /* United States */
+	CTRY_UNITED_STATES_FCC49  = 842,     /* United States (Public Safety)*/
+	CTRY_URUGUAY              = 858,     /* Uruguay */
+	CTRY_UZBEKISTAN           = 860,     /* Uzbekistan */
+	CTRY_VENEZUELA            = 862,     /* Venezuela */
+	CTRY_VIET_NAM             = 704,     /* Viet Nam */
+	CTRY_YEMEN                = 887,     /* Yemen */
+	CTRY_ZIMBABWE             = 716      /* Zimbabwe */
 };
 
 /* 
@@ -358,7 +358,7 @@ enum CountryCode {
 struct ieee80211_ie {
 	u_int8_t id;
 	u_int8_t len;
-	u_int8_t info[];
+	u_int8_t info[0];
 } __packed;
 
 /* 
@@ -387,7 +387,7 @@ struct ieee80211_ie_csa {
 #define IEEE80211_CSA_PROTECTION_PERIOD 3
 
 /* maximum allowed deviance of measurement of intervals between CSA in Beacons */
-#define IEEE80211_CSA_SANITY_THRESHOLD 100    
+#define IEEE80211_CSA_SANITY_THRESHOLD 100
 
 
 /* does frame have QoS sequence control data */
@@ -514,7 +514,7 @@ struct ieee80211_ie_athAdvCap {
 	u_int8_t athAdvCap_id;		/* IEEE80211_ELEMID_VENDOR */
 	u_int8_t athAdvCap_len;		/* length in bytes */
 	u_int8_t athAdvCap_oui[3];	/* 0x00, 0x03, 0x7f */
-	u_int8_t athAdvCap_type;		/* OUI type */
+	u_int8_t athAdvCap_type;	/* OUI type */
 	u_int8_t athAdvCap_subtype;	/* OUI subtype */
 	u_int8_t athAdvCap_version;	/* spec revision */
 	u_int8_t athAdvCap_capability;	/* Capability info */
@@ -567,7 +567,7 @@ struct ieee80211_mnf {
  */
 struct ieee80211_frame_min {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_addr1[IEEE80211_ADDR_LEN];
 	u_int8_t i_addr2[IEEE80211_ADDR_LEN];
 	/* FCS */
@@ -575,7 +575,7 @@ struct ieee80211_frame_min {
 
 struct ieee80211_frame_rts {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_ra[IEEE80211_ADDR_LEN];
 	u_int8_t i_ta[IEEE80211_ADDR_LEN];
 	/* FCS */
@@ -583,14 +583,14 @@ struct ieee80211_frame_rts {
 
 struct ieee80211_frame_cts {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_ra[IEEE80211_ADDR_LEN];
 	/* FCS */
 } __packed;
 
 struct ieee80211_frame_ack {
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];
+	__le16 i_dur;
 	u_int8_t i_ra[IEEE80211_ADDR_LEN];
 	/* FCS */
 } __packed;
@@ -605,7 +605,7 @@ struct ieee80211_frame_pspoll {
 
 struct ieee80211_frame_cfend {		/* NB: also CF-End+CF-Ack */
 	u_int8_t i_fc[2];
-	u_int8_t i_dur[2];	/* should be zero */
+	__le16 i_dur;	/* should be zero */
 	u_int8_t i_ra[IEEE80211_ADDR_LEN];
 	u_int8_t i_bssid[IEEE80211_ADDR_LEN];
 	/* FCS */
@@ -638,7 +638,7 @@ typedef u_int8_t *ieee80211_mgt_beacon_t;
 #define	IEEE80211_CAPINFO_SHORT_PREAMBLE	0x0020
 #define	IEEE80211_CAPINFO_PBCC			0x0040
 #define	IEEE80211_CAPINFO_CHNL_AGILITY		0x0080
-/* bits 8-9 are reserved (8 now for specturm management) */
+/* bits 8-9 are reserved (8 now for spectrum management) */
 #define IEEE80211_CAPINFO_SPECTRUM_MGMT		0x0100
 #define	IEEE80211_CAPINFO_SHORT_SLOTTIME	0x0400
 #define	IEEE80211_CAPINFO_RSN			0x0800
@@ -662,7 +662,7 @@ struct ieee80211_ie_wpa {
 	u_int32_t wpa_authsels[8];	/* selectors */
 	u_int16_t wpa_caps;		/* 802.11i capabilities */
 	u_int16_t wpa_pmkidcnt;		/* 802.11i pmkid count */
-	u_int16_t wpa_pmkids[8];		/* 802.11i pmkids */
+	u_int16_t wpa_pmkids[8];	/* 802.11i pmkids */
 } __packed;
 
 /*
@@ -678,7 +678,14 @@ enum {
 	IEEE80211_ELEMID_TIM		= 5,
 	IEEE80211_ELEMID_IBSSPARMS	= 6,
 	IEEE80211_ELEMID_COUNTRY	= 7,
+	IEEE80211_ELEMID_HOPPATRNPARMS  = 8,
+	IEEE80211_ELEMID_HOPPATRNTABLE  = 9,
 	IEEE80211_ELEMID_REQINFO	= 10,
+	IEEE80211_ELEMID_QBSSLOAD       = 11,
+	IEEE80211_ELEMID_EDCAPARAM      = 12,
+	IEEE80211_ELEMID_TSPEC		= 13,
+	IEEE80211_ELEMID_TRAFCLASS      = 14,
+	IEEE80211_ELEMID_SCHEDULE       = 15,
 	IEEE80211_ELEMID_CHALLENGE	= 16,
 	/* 17-31 reserved for challenge text extension */
 	IEEE80211_ELEMID_PWRCNSTR	= 32,
@@ -692,7 +699,11 @@ enum {
 	IEEE80211_ELEMID_QUIET		= 40,
 	IEEE80211_ELEMID_IBSSDFS	= 41,
 	IEEE80211_ELEMID_ERP		= 42,
+	IEEE80211_ELEMID_TSDELAY	= 43,
+	IEEE80211_ELEMID_TCLASSPROC     = 44,
+	IEEE80211_ELEMID_QOS		= 46,
 	IEEE80211_ELEMID_RSN		= 48,
+	/* 49 reserved */
 	IEEE80211_ELEMID_XRATES		= 50,
 	/* 128-129 proprietary elements used by Agere chipsets */
 	IEEE80211_ELEMID_AGERE1		= 128,
@@ -746,9 +757,10 @@ struct ieee80211_country_ie {
 #define	ATH_OUI			0x7f0300		/* Atheros OUI */
 #define	ATH_OUI_TYPE		0x01
 #define	ATH_OUI_SUBTYPE		0x01
-#define ATH_OUI_VERSION		0x00
+#define	ATH_OUI_VERSION		0x00
 #define	ATH_OUI_TYPE_XR		0x03
-#define	ATH_OUI_VER_XR		0x01
+#define	ATH_OUI_SUBTYPE_XR	0x01
+#define	ATH_OUI_VER_XR		0x00
 
 #define	WPA_OUI			0xf25000
 #define	WPA_OUI_TYPE		0x01
@@ -845,6 +857,8 @@ enum {
 	IEEE80211_REASON_ASSOC_LEAVE		= 8,
 	IEEE80211_REASON_ASSOC_NOT_AUTHED	= 9,
 
+	IEEE80211_REASON_PWRCAP_UNACCEPTABLE	= 10,
+	IEEE80211_REASON_SUPPCHAN_UNACCEPTABLE	= 11,
 	IEEE80211_REASON_RSN_REQUIRED		= 11,
 	IEEE80211_REASON_RSN_INCONSISTENT	= 12,
 	IEEE80211_REASON_IE_INVALID		= 13,
@@ -864,6 +878,9 @@ enum {
 	IEEE80211_STATUS_SP_REQUIRED		= 19,
 	IEEE80211_STATUS_PBCC_REQUIRED		= 20,
 	IEEE80211_STATUS_CA_REQUIRED		= 21,
+	IEEE80211_STATUS_SM_REQUIRED		= 22,
+	IEEE80211_STATUS_PWRCAP_UNACCEPTABLE	= 23,
+	IEEE80211_STATUS_SUPPCHAN_UNACCEPTABLE	= 24,
 	IEEE80211_STATUS_TOO_MANY_STATIONS	= 22,
 	IEEE80211_STATUS_RATES			= 23,
 	IEEE80211_STATUS_SHORTSLOT_REQUIRED	= 25,
@@ -900,7 +917,7 @@ enum {
 #define	IEEE80211_MTU_MIN		32
 
 #define	IEEE80211_MAX_LEN		(2300 + IEEE80211_CRC_LEN + \
-    (IEEE80211_WEP_IVLEN + IEEE80211_WEP_KIDLEN + IEEE80211_WEP_CRCLEN))
+	(IEEE80211_WEP_IVLEN + IEEE80211_WEP_KIDLEN + IEEE80211_WEP_CRCLEN))
 #define	IEEE80211_ACK_LEN \
 	(sizeof(struct ieee80211_frame_ack) + IEEE80211_CRC_LEN)
 #define	IEEE80211_MIN_LEN \
@@ -908,7 +925,7 @@ enum {
 
 /*
  * The 802.11 spec says at most 2007 stations may be
- * associated at once.  For most AP's this is way more
+ * associated at once.  For most APs this is way more
  * than is feasible so we use a default of 128.  This
  * number may be overridden by the driver and/or by
  * user configuration.
@@ -927,7 +944,7 @@ enum {
 #define	IEEE80211_RTS_MAX		2346
 
 /* 
- * Regulatory extention identifier for country IE.
+ * Regulatory extension identifier for country IE.
  */
 #define IEEE80211_REG_EXT_ID		201
 
