@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_beacon.c 1756 2006-10-18 11:41:05Z kelmo $
+ * $Id: ieee80211_beacon.c 2902 2007-11-21 20:14:11Z mtaylor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -50,13 +50,6 @@
 #include "if_media.h"
 #include <net80211/ieee80211_var.h>
 
-/* 
- *  XXX: Include an intra-module function from ieee80211_input.c.
- *       When we move regdomain code out to separate .h/.c files
- *       this should go to that .h file.
- */
-struct ieee80211_channel *ieee80211_doth_findchan(struct ieee80211vap *, u_int8_t);
-
 
 static u_int8_t *
 ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets *bo,
@@ -74,7 +67,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 	frm += 8;
 
 	/* beacon interval */
-	*(u_int16_t *)frm = htole16(ni->ni_intval);
+	*(__le16 *)frm = htole16(ni->ni_intval);
 	frm += 2;
 
 	/* capability information */
@@ -91,8 +84,8 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 		capinfo |= IEEE80211_CAPINFO_SHORT_SLOTTIME;
 	if (ic->ic_flags & IEEE80211_F_DOTH)
 		capinfo |= IEEE80211_CAPINFO_SPECTRUM_MGMT;
-	bo->bo_caps = (u_int16_t *)frm;
-	*(u_int16_t *)frm = htole16(capinfo);
+	bo->bo_caps = (__le16 *)frm;
+	*(__le16 *)frm = htole16(capinfo);
 	frm += 2;
 
 	/* ssid */
@@ -103,7 +96,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 		frm += ni->ni_esslen;
 	} else
 		*frm++ = 0;
-	
+
 	/* supported rates */
 	frm = ieee80211_add_rates(frm, rs);
 
@@ -138,7 +131,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 	bo->bo_tim_trailer = frm;
 
 	/* country */
-	if ((ic->ic_flags & IEEE80211_F_DOTH) || 
+	if ((ic->ic_flags & IEEE80211_F_DOTH) ||
 	    (ic->ic_flags_ext & IEEE80211_FEXT_COUNTRYIE)) {
 		frm = ieee80211_add_country(frm, ic);
 	}
@@ -150,7 +143,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 		*frm++ = IEEE80211_PWRCONSTRAINT_VAL(ic);
 	}
 
-	/* XXX: channel switch announcement ? */ 
+	/* XXX: channel switch announcement ? */
 	bo->bo_chanswitch = frm;
 
 	/* ERP */
@@ -178,7 +171,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 	if (vap->iv_bss && vap->iv_bss->ni_ath_flags)
 		frm = ieee80211_add_athAdvCap(frm, vap->iv_bss->ni_ath_flags,
 			vap->iv_bss->ni_ath_defkeyindex);
-	
+
 	/* XR */
 	bo->bo_xr = frm;
 #ifdef ATH_SUPERG_XR
@@ -187,7 +180,7 @@ ieee80211_beacon_init(struct ieee80211_node *ni, struct ieee80211_beacon_offsets
 #endif
 	bo->bo_appie_buf = frm;
 	bo->bo_appie_buf_len = 0;
-	
+
 	bo->bo_tim_trailerlen = frm - bo->bo_tim_trailer;
 	bo->bo_chanswitch_trailerlen = frm - bo->bo_chanswitch;
 
@@ -214,7 +207,7 @@ ieee80211_beacon_alloc(struct ieee80211_node *ni,
 	 * beacon frame format
 	 *	[8] time stamp
 	 *	[2] beacon interval
-	 *	[2] cabability information
+	 *	[2] capability information
 	 *	[tlv] ssid
 	 *	[tlv] supported rates
 	 *	[7] FH/DS parameter set
@@ -232,18 +225,18 @@ ieee80211_beacon_alloc(struct ieee80211_node *ni,
 	 * NB: we allocate the max space required for the TIM bitmap.
 	 */
 	rs = &ni->ni_rates;
-	pktlen =   8					/* time stamp */
+	pktlen = 8					/* time stamp */
 		 + sizeof(u_int16_t)			/* beacon interval */
 		 + sizeof(u_int16_t)			/* capability information */
 		 + 2 + ni->ni_esslen			/* ssid */
-	         + 2 + IEEE80211_RATE_SIZE		/* supported rates */
-	         + 7 					/* FH/DS parameters max(7,3) */
+		 + 2 + IEEE80211_RATE_SIZE		/* supported rates */
+		 + 7 					/* FH/DS parameters max(7,3) */
 		 + 2 + 4 + vap->iv_tim_len		/* IBSS/TIM parameter set*/
 		 + ic->ic_country_ie.country_len + 2	/* country code */
-	         + 3					/* power constraint */
-	         + 5					/* channel switch announcement */
+		 + 3					/* power constraint */
+		 + 5					/* channel switch announcement */
 		 + 3					/* ERP */
-	         + 2 + (IEEE80211_RATE_MAXSIZE - IEEE80211_RATE_SIZE) /* Ext. Supp. Rates */
+		 + 2 + (IEEE80211_RATE_MAXSIZE - IEEE80211_RATE_SIZE) /* Ext. Supp. Rates */
 		 + (vap->iv_caps & IEEE80211_C_WME ?	/* WME */
 			sizeof(struct ieee80211_wme_param) : 0)
 		 + (vap->iv_caps & IEEE80211_C_WPA ?	/* WPA 1+2 */
@@ -262,6 +255,8 @@ ieee80211_beacon_alloc(struct ieee80211_node *ni,
 		return NULL;
 	}
 
+	SKB_CB(skb)->ni = ieee80211_ref_node(ni);
+
 	frm = ieee80211_beacon_init(ni, bo, frm);
 
 	skb_trim(skb, frm - skb->data);
@@ -271,7 +266,7 @@ ieee80211_beacon_alloc(struct ieee80211_node *ni,
 	wh->i_fc[0] = IEEE80211_FC0_VERSION_0 | IEEE80211_FC0_TYPE_MGT |
 		IEEE80211_FC0_SUBTYPE_BEACON;
 	wh->i_fc[1] = IEEE80211_FC1_DIR_NODS;
-	*(u_int16_t *)wh->i_dur = 0;
+	wh->i_dur = 0;
 	IEEE80211_ADDR_COPY(wh->i_addr1, ic->ic_dev->broadcast);
 	IEEE80211_ADDR_COPY(wh->i_addr2, vap->iv_myaddr);
 	IEEE80211_ADDR_COPY(wh->i_addr3, ni->ni_bssid);
@@ -293,47 +288,53 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 	int len_changed = 0;
 	u_int16_t capinfo;
 
-	IEEE80211_LOCK(ic);
+	IEEE80211_LOCK_IRQ(ic);
+
+	/* Check if we need to change channel right now */
 
 	if ((ic->ic_flags & IEEE80211_F_DOTH) &&
-	    (vap->iv_flags & IEEE80211_F_CHANSWITCH) &&
-	    (vap->iv_chanchange_count == ic->ic_chanchange_tbtt)) {
-		u_int8_t *frm;
-		struct ieee80211_channel *c;
+	    (vap->iv_flags & IEEE80211_F_CHANSWITCH)) {
+		struct ieee80211_channel *c = 
+			ieee80211_doth_findchan(vap, ic->ic_chanchange_chan);
+		
+		if (!vap->iv_chanchange_count && !c) {
+			vap->iv_flags &= ~IEEE80211_F_CHANSWITCH;
+			ic->ic_flags &= ~IEEE80211_F_CHANSWITCH;
+		} else if (vap->iv_chanchange_count == ic->ic_chanchange_tbtt) {
+			u_int8_t *frm;
 
-		vap->iv_chanchange_count = 0;
-
-		IEEE80211_DPRINTF(vap, IEEE80211_MSG_DOTH,
-			"%s: reinit beacon\n", __func__);
-
-		/* 
-		 * NB: ic_bsschan is in the DSPARMS beacon IE, so must set this
-		 *     prior to the beacon re-init, below.
-		 */
-		c = ieee80211_doth_findchan(vap, ic->ic_chanchange_chan);
-		if (c == NULL) {
 			IEEE80211_DPRINTF(vap, IEEE80211_MSG_DOTH,
-				"%s: find channel failure\n", __func__);
-			return 0;
+					"%s: reinit beacon\n", __func__);
+
+			/* NB: ic_bsschan is in the DSPARMS beacon IE, so must
+			 * set this prior to the beacon re-init, below. */
+			if (c == NULL) {
+				/* Requested channel invalid; drop the channel
+				 * switch announcement and do nothing. */
+				IEEE80211_DPRINTF(vap, IEEE80211_MSG_DOTH,
+						"%s: find channel failure\n", __func__);
+			} else
+				ic->ic_bsschan = c;
+
+			skb_pull(skb, sizeof(struct ieee80211_frame));
+			skb_trim(skb, 0);
+			frm = skb->data;
+			skb_put(skb, ieee80211_beacon_init(ni, bo, frm) - frm);
+			skb_push(skb, sizeof(struct ieee80211_frame));
+
+			vap->iv_chanchange_count = 0;
+			vap->iv_flags &= ~IEEE80211_F_CHANSWITCH;
+			ic->ic_flags &= ~IEEE80211_F_CHANSWITCH;
+
+			/* NB: Only for the first VAP to get here, and when we
+			 * have a valid channel to which to change. */
+			if (c && (ic->ic_curchan != c)) {
+				ic->ic_curchan = c;
+				ic->ic_set_channel(ic);
+			}
+
+			len_changed = 1;
 		}
-		ic->ic_bsschan = c;
-
-		skb_pull(skb, sizeof(struct ieee80211_frame));
-		skb_trim(skb, 0);
-		frm = skb->data;
-		skb_put(skb, ieee80211_beacon_init(ni, bo, frm) - frm); 
-		skb_push(skb, sizeof(struct ieee80211_frame));
-
-		vap->iv_flags &= ~IEEE80211_F_CHANSWITCH;
-		ic->ic_flags &= ~IEEE80211_F_CHANSWITCH;
-
-		/* NB: only for the first VAP to get here */
-		if (ic->ic_curchan != c) {
-			ic->ic_curchan = c;
-			ic->ic_set_channel(ic);
-		}
-
-		len_changed = 1;
 	}
 
 	/* XXX faster to recalculate entirely or just changes? */
@@ -341,6 +342,7 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 		capinfo = IEEE80211_CAPINFO_IBSS;
 	else
 		capinfo = IEEE80211_CAPINFO_ESS;
+
 	if (vap->iv_flags & IEEE80211_F_PRIVACY)
 		capinfo |= IEEE80211_CAPINFO_PRIVACY;
 	if ((ic->ic_flags & IEEE80211_F_SHPREAMBLE) &&
@@ -357,10 +359,10 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 		struct ieee80211_wme_state *wme = &ic->ic_wme;
 
 		/*
-		 * Check for agressive mode change.  When there is
+		 * Check for aggressive mode change.  When there is
 		 * significant high priority traffic in the BSS
 		 * throttle back BE traffic by using conservative
-		 * parameters.  Otherwise BE uses agressive params
+		 * parameters.  Otherwise BE uses aggressive params
 		 * to optimize performance of legacy/non-QoS traffic.
 		 */
 		if (wme->wme_flags & WME_F_AGGRMODE) {
@@ -482,7 +484,7 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 				vap->iv_flags |= IEEE80211_F_CHANSWITCH;
 
 				/* copy out trailer to open up a slot */
-				memmove(bo->bo_chanswitch + IEEE80211_CHANSWITCHANN_BYTES, 
+				memmove(bo->bo_chanswitch + IEEE80211_CHANSWITCHANN_BYTES,
 					bo->bo_chanswitch, bo->bo_chanswitch_trailerlen);
 
 				/* add ie in opened slot */
@@ -503,9 +505,9 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 				/* indicate new beacon length so other layers may manage memory */
 				skb_put(skb, IEEE80211_CHANSWITCHANN_BYTES);
 				len_changed = 1;
-			}
-			else
+			} else
 				bo->bo_chanswitch[4]--;
+			
 			vap->iv_chanchange_count++;
 			IEEE80211_DPRINTF(vap, IEEE80211_MSG_DOTH,
 				"%s: CHANSWITCH IE, change in %d\n",
@@ -518,7 +520,7 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 			vap->iv_flags &= ~IEEE80211_F_XRUPDATE;
 		}
 #endif
-		if (ic->ic_flags_ext & IEEE80211_FEXT_ERPUPDATE) {
+		if ((ic->ic_flags_ext & IEEE80211_FEXT_ERPUPDATE) && (bo->bo_erp != NULL)) {
 			(void) ieee80211_add_erp(bo->bo_erp, ic);
 			ic->ic_flags_ext &= ~IEEE80211_FEXT_ERPUPDATE;
 		}
@@ -547,13 +549,13 @@ ieee80211_beacon_update(struct ieee80211_node *ni,
 
 			len_changed = 1;
 		}
-		memcpy(bo->bo_appie_buf,vap->app_ie[IEEE80211_APPIE_FRAME_BEACON].ie,
+		memcpy(bo->bo_appie_buf, vap->app_ie[IEEE80211_APPIE_FRAME_BEACON].ie,
 			vap->app_ie[IEEE80211_APPIE_FRAME_BEACON].length);
 
 		vap->iv_flags_ext &= ~IEEE80211_FEXT_APPIE_UPDATE;
 	}
 
-	IEEE80211_UNLOCK(ic);
+	IEEE80211_UNLOCK_IRQ(ic);
 
 	return len_changed;
 }
