@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_ioctl.h 1756 2006-10-18 11:41:05Z kelmo $
+ * $Id: ieee80211_ioctl.h 2889 2007-11-21 11:08:51Z mtaylor $
  */
 #ifndef _NET80211_IEEE80211_IOCTL_H_
 #define _NET80211_IEEE80211_IOCTL_H_
@@ -205,7 +205,7 @@ struct ieee80211_stats {
 struct ieee80211req_key {
 	u_int8_t ik_type;		/* key/cipher type */
 	u_int8_t ik_pad;
-	u_int16_t ik_keyix;		/* key index */
+	ieee80211_keyix_t ik_keyix;	/* key index */
 	u_int8_t ik_keylen;		/* key length in bytes */
 	u_int8_t ik_flags;
 /* NB: IEEE80211_KEY_XMIT and IEEE80211_KEY_RECV defined elsewhere */
@@ -221,7 +221,8 @@ struct ieee80211req_key {
  * to IEEE80211_KEYIX_NONE when deleting a unicast key.
  */
 struct ieee80211req_del_key {
-	u_int8_t idk_keyix;		/* key index */
+	/* XXX: This should be ieee80211_keyix_t, but it changes API/ABI for hostapd */
+	u_int8_t idk_keyix;	/* key index */
 	u_int8_t idk_macaddr[IEEE80211_ADDR_LEN];
 };
 
@@ -249,7 +250,7 @@ struct ieee80211req_mlme {
  * MAC ACL operations.
  */
 enum {
-	IEEE80211_MACCMD_POLICY_OPEN	= 0,	/* set policy: no ACL's */
+	IEEE80211_MACCMD_POLICY_OPEN	= 0,	/* set policy: no ACLs */
 	IEEE80211_MACCMD_POLICY_ALLOW	= 1,	/* set policy: allow traffic */
 	IEEE80211_MACCMD_POLICY_DENY	= 2,	/* set policy: deny traffic */
 	IEEE80211_MACCMD_FLUSH		= 3,	/* flush ACL database */
@@ -507,6 +508,21 @@ struct ieee80211req_scan_result {
 #ifdef __KERNEL__
 #include <linux/if.h>
 #endif
+/* The maximum size of a iwpriv structure is IW_PRIV_SIZE_MASK, which was 
+ * exceeded for some time by chaninfo ioctl.  These macros change the size 
+ * encoding for anything larger than IW_PRIV_SIZE_MASK from bytes to 4-byte
+ * multiples so that the padded size fits under IW_PRIV_SIZE_MASK. */
+#define IW_PRIV_BLOB_LENGTH_ENCODING(_SIZE) \
+	(((_SIZE) == ((_SIZE) & IW_PRIV_SIZE_MASK)) ? \
+		(_SIZE) : \
+		(((_SIZE) / sizeof(uint32_t)) + \
+			(((_SIZE) == (((_SIZE) / sizeof(uint32_t)) * sizeof(int))) ? \
+				0 : 1)))
+#define IW_PRIV_BLOB_TYPE_ENCODING(_SIZE) \
+	(((_SIZE) == ((_SIZE) & IW_PRIV_SIZE_MASK)) ? \
+		(IW_PRIV_TYPE_BYTE | (_SIZE)) : \
+		(IW_PRIV_TYPE_INT  | IW_PRIV_BLOB_LENGTH_ENCODING((_SIZE))))
+
 #define	IEEE80211_IOCTL_SETPARAM	(SIOCIWFIRSTPRIV+0)
 #define	IEEE80211_IOCTL_GETPARAM	(SIOCIWFIRSTPRIV+1)
 #define	IEEE80211_IOCTL_SETMODE		(SIOCIWFIRSTPRIV+2)
@@ -518,12 +534,15 @@ struct ieee80211req_scan_result {
 #define	IEEE80211_IOCTL_CHANSWITCH	(SIOCIWFIRSTPRIV+8)
 #define	IEEE80211_IOCTL_GET_APPIEBUF	(SIOCIWFIRSTPRIV+9)
 #define	IEEE80211_IOCTL_SET_APPIEBUF	(SIOCIWFIRSTPRIV+10)
+#define	IEEE80211_IOCTL_READREG		(SIOCIWFIRSTPRIV+11)
 #define	IEEE80211_IOCTL_FILTERFRAME	(SIOCIWFIRSTPRIV+12)
 #define	IEEE80211_IOCTL_GETCHANINFO	(SIOCIWFIRSTPRIV+13)
 #define	IEEE80211_IOCTL_SETOPTIE	(SIOCIWFIRSTPRIV+14)
 #define	IEEE80211_IOCTL_GETOPTIE	(SIOCIWFIRSTPRIV+15)
 #define	IEEE80211_IOCTL_SETMLME		(SIOCIWFIRSTPRIV+16)
+#define	IEEE80211_IOCTL_RADAR		(SIOCIWFIRSTPRIV+17)
 #define	IEEE80211_IOCTL_SETKEY		(SIOCIWFIRSTPRIV+18)
+#define	IEEE80211_IOCTL_WRITEREG	(SIOCIWFIRSTPRIV+19)
 #define	IEEE80211_IOCTL_DELKEY		(SIOCIWFIRSTPRIV+20)
 #define	IEEE80211_IOCTL_ADDMAC		(SIOCIWFIRSTPRIV+22)
 #define	IEEE80211_IOCTL_DELMAC		(SIOCIWFIRSTPRIV+24)
@@ -537,67 +556,83 @@ enum {
 	IEEE80211_WMMPARAMS_AIFS       	= 3,
 	IEEE80211_WMMPARAMS_TXOPLIMIT	= 4,
 	IEEE80211_WMMPARAMS_ACM		= 5,
-	IEEE80211_WMMPARAMS_NOACKPOLICY	= 6,	
+	IEEE80211_WMMPARAMS_NOACKPOLICY	= 6,
 };
 enum {
-	IEEE80211_PARAM_TURBO		= 1,	/* turbo mode */
-	IEEE80211_PARAM_MODE		= 2,	/* phy mode (11a, 11b, etc.) */
-	IEEE80211_PARAM_AUTHMODE	= 3,	/* authentication mode */
-	IEEE80211_PARAM_PROTMODE	= 4,	/* 802.11g protection */
-	IEEE80211_PARAM_MCASTCIPHER	= 5,	/* multicast/default cipher */
-	IEEE80211_PARAM_MCASTKEYLEN	= 6,	/* multicast key length */
-	IEEE80211_PARAM_UCASTCIPHERS	= 7,	/* unicast cipher suites */
-	IEEE80211_PARAM_UCASTCIPHER	= 8,	/* unicast cipher */
-	IEEE80211_PARAM_UCASTKEYLEN	= 9,	/* unicast key length */
-	IEEE80211_PARAM_WPA		= 10,	/* WPA mode (0,1,2) */
-	IEEE80211_PARAM_ROAMING		= 12,	/* roaming mode */
-	IEEE80211_PARAM_PRIVACY		= 13,	/* privacy invoked */
-	IEEE80211_PARAM_COUNTERMEASURES	= 14,	/* WPA/TKIP countermeasures */
-	IEEE80211_PARAM_DROPUNENCRYPTED	= 15,	/* discard unencrypted frames */
-	IEEE80211_PARAM_DRIVER_CAPS	= 16,	/* driver capabilities */
-	IEEE80211_PARAM_MACCMD		= 17,	/* MAC ACL operation */
-	IEEE80211_PARAM_WMM		= 18,	/* WMM mode (on, off) */
-	IEEE80211_PARAM_HIDESSID	= 19,	/* hide SSID mode (on, off) */
-	IEEE80211_PARAM_APBRIDGE    	= 20,   /* AP inter-sta bridging */
-	IEEE80211_PARAM_KEYMGTALGS	= 21,	/* key management algorithms */
-	IEEE80211_PARAM_RSNCAPS		= 22,	/* RSN capabilities */
-	IEEE80211_PARAM_INACT		= 23,	/* station inactivity timeout */
-	IEEE80211_PARAM_INACT_AUTH	= 24,	/* station auth inact timeout */
-	IEEE80211_PARAM_INACT_INIT	= 25,	/* station init inact timeout */
-	IEEE80211_PARAM_ABOLT		= 26,	/* Atheros Adv. Capabilities */
-	IEEE80211_PARAM_DTIM_PERIOD	= 28,	/* DTIM period (beacons) */
-	IEEE80211_PARAM_BEACON_INTERVAL	= 29,	/* beacon interval (ms) */
-	IEEE80211_PARAM_DOTH		= 30,	/* 11.h is on/off */
-	IEEE80211_PARAM_PWRTARGET	= 31,	/* Current Channel Pwr Constraint */
-	IEEE80211_PARAM_GENREASSOC	= 32,	/* Generate a reassociation request */
-	IEEE80211_PARAM_COMPRESSION	= 33,	/* compression */
-	IEEE80211_PARAM_FF		= 34,	/* fast frames support  */
-	IEEE80211_PARAM_XR		= 35,	/* XR support */
-	IEEE80211_PARAM_BURST		= 36,	/* burst mode */
-	IEEE80211_PARAM_PUREG		= 37,	/* pure 11g (no 11b stations) */
-	IEEE80211_PARAM_AR		= 38,	/* AR support */
-	IEEE80211_PARAM_WDS		= 39,	/* Enable 4 address processing */
-	IEEE80211_PARAM_BGSCAN		= 40,	/* bg scanning (on, off) */
-	IEEE80211_PARAM_BGSCAN_IDLE	= 41,	/* bg scan idle threshold */
-	IEEE80211_PARAM_BGSCAN_INTERVAL	= 42,	/* bg scan interval */
-	IEEE80211_PARAM_MCAST_RATE	= 43,	/* Multicast Tx Rate */
-	IEEE80211_PARAM_COVERAGE_CLASS	= 44,	/* coverage class */
-	IEEE80211_PARAM_COUNTRY_IE	= 45,	/* enable country IE */
-	IEEE80211_PARAM_SCANVALID	= 46,	/* scan cache valid threshold */
-	IEEE80211_PARAM_ROAM_RSSI_11A	= 47,	/* rssi threshold in 11a */
-	IEEE80211_PARAM_ROAM_RSSI_11B	= 48,	/* rssi threshold in 11b */
-	IEEE80211_PARAM_ROAM_RSSI_11G	= 49,	/* rssi threshold in 11g */
-	IEEE80211_PARAM_ROAM_RATE_11A	= 50,	/* tx rate threshold in 11a */
-	IEEE80211_PARAM_ROAM_RATE_11B	= 51,	/* tx rate threshold in 11b */
-	IEEE80211_PARAM_ROAM_RATE_11G	= 52,	/* tx rate threshold in 11g */
-	IEEE80211_PARAM_UAPSDINFO	= 53,	/* value for qos info field */
-	IEEE80211_PARAM_SLEEP		= 54,	/* force sleep/wake */
-	IEEE80211_PARAM_QOSNULL		= 55,	/* force sleep/wake */
-	IEEE80211_PARAM_PSPOLL		= 56,	/* force ps-poll generation (sta only) */
-	IEEE80211_PARAM_EOSPDROP	= 57,	/* force uapsd EOSP drop (ap only) */
-	IEEE80211_PARAM_MARKDFS		= 58,	/* mark a dfs interference channel when found */
-	IEEE80211_PARAM_REGCLASS	= 59,	/* enable regclass ids in country IE */
-	IEEE80211_PARAM_DROPUNENC_EAPOL	= 60,	/* drop unencrypted eapol frames */
+	IEEE80211_PARAM_TURBO			= 1,	/* turbo mode */
+	IEEE80211_PARAM_MODE			= 2,	/* phy mode (11a, 11b, etc.) */
+	IEEE80211_PARAM_AUTHMODE		= 3,	/* authentication mode */
+	IEEE80211_PARAM_PROTMODE		= 4,	/* 802.11g protection */
+	IEEE80211_PARAM_MCASTCIPHER		= 5,	/* multicast/default cipher */
+	IEEE80211_PARAM_MCASTKEYLEN		= 6,	/* multicast key length */
+	IEEE80211_PARAM_UCASTCIPHERS		= 7,	/* unicast cipher suites */
+	IEEE80211_PARAM_UCASTCIPHER		= 8,	/* unicast cipher */
+	IEEE80211_PARAM_UCASTKEYLEN		= 9,	/* unicast key length */
+	IEEE80211_PARAM_WPA			= 10,	/* WPA mode (0,1,2) */
+	IEEE80211_PARAM_ROAMING			= 12,	/* roaming mode */
+	IEEE80211_PARAM_PRIVACY			= 13,	/* privacy invoked */
+	IEEE80211_PARAM_COUNTERMEASURES		= 14,	/* WPA/TKIP countermeasures */
+	IEEE80211_PARAM_DROPUNENCRYPTED		= 15,	/* discard unencrypted frames */
+	IEEE80211_PARAM_DRIVER_CAPS		= 16,	/* driver capabilities */
+	IEEE80211_PARAM_MACCMD			= 17,	/* MAC ACL operation */
+	IEEE80211_PARAM_WMM			= 18,	/* WMM mode (on, off) */
+	IEEE80211_PARAM_HIDESSID		= 19,	/* hide SSID mode (on, off) */
+	IEEE80211_PARAM_APBRIDGE    		= 20,   /* AP inter-sta bridging */
+	IEEE80211_PARAM_KEYMGTALGS		= 21,	/* key management algorithms */
+	IEEE80211_PARAM_RSNCAPS			= 22,	/* RSN capabilities */
+	IEEE80211_PARAM_INACT			= 23,	/* station inactivity timeout */
+	IEEE80211_PARAM_INACT_AUTH		= 24,	/* station auth inact timeout */
+	IEEE80211_PARAM_INACT_INIT		= 25,	/* station init inact timeout */
+	IEEE80211_PARAM_ABOLT			= 26,	/* Atheros Adv. Capabilities */
+	IEEE80211_PARAM_DTIM_PERIOD		= 28,	/* DTIM period (beacons) */
+	IEEE80211_PARAM_BEACON_INTERVAL		= 29,	/* beacon interval (ms) */
+	IEEE80211_PARAM_DOTH			= 30,	/* 11.h is on/off */
+	IEEE80211_PARAM_PWRTARGET		= 31,	/* Current Channel Pwr Constraint */
+	IEEE80211_PARAM_GENREASSOC		= 32,	/* Generate a reassociation request */
+	IEEE80211_PARAM_COMPRESSION		= 33,	/* compression */
+	IEEE80211_PARAM_FF			= 34,	/* fast frames support  */
+	IEEE80211_PARAM_XR			= 35,	/* XR support */
+	IEEE80211_PARAM_BURST			= 36,	/* burst mode */
+	IEEE80211_PARAM_PUREG			= 37,	/* pure 11g (no 11b stations) */
+	IEEE80211_PARAM_AR			= 38,	/* AR support */
+	IEEE80211_PARAM_WDS			= 39,	/* Enable 4 address processing */
+	IEEE80211_PARAM_BGSCAN			= 40,	/* bg scanning (on, off) */
+	IEEE80211_PARAM_BGSCAN_IDLE		= 41,	/* bg scan idle threshold */
+	IEEE80211_PARAM_BGSCAN_INTERVAL		= 42,	/* bg scan interval */
+	IEEE80211_PARAM_MCAST_RATE		= 43,	/* Multicast Tx Rate */
+	IEEE80211_PARAM_COVERAGE_CLASS		= 44,	/* coverage class */
+	IEEE80211_PARAM_COUNTRY_IE		= 45,	/* enable country IE */
+	IEEE80211_PARAM_SCANVALID		= 46,	/* scan cache valid threshold */
+	IEEE80211_PARAM_ROAM_RSSI_11A		= 47,	/* rssi threshold in 11a */
+	IEEE80211_PARAM_ROAM_RSSI_11B		= 48,	/* rssi threshold in 11b */
+	IEEE80211_PARAM_ROAM_RSSI_11G		= 49,	/* rssi threshold in 11g */
+	IEEE80211_PARAM_ROAM_RATE_11A		= 50,	/* tx rate threshold in 11a */
+	IEEE80211_PARAM_ROAM_RATE_11B		= 51,	/* tx rate threshold in 11b */
+	IEEE80211_PARAM_ROAM_RATE_11G		= 52,	/* tx rate threshold in 11g */
+	IEEE80211_PARAM_UAPSDINFO		= 53,	/* value for qos info field */
+	IEEE80211_PARAM_SLEEP			= 54,	/* force sleep/wake */
+	IEEE80211_PARAM_QOSNULL			= 55,	/* force sleep/wake */
+	IEEE80211_PARAM_PSPOLL			= 56,	/* force ps-poll generation (sta only) */
+	IEEE80211_PARAM_EOSPDROP		= 57,	/* force uapsd EOSP drop (ap only) */
+	IEEE80211_PARAM_MARKDFS			= 58,	/* mark a dfs interference channel when found */
+	IEEE80211_PARAM_REGCLASS		= 59,	/* enable regclass ids in country IE */
+	IEEE80211_PARAM_DROPUNENC_EAPOL		= 60,	/* drop unencrypted eapol frames */
+	IEEE80211_PARAM_SHPREAMBLE		= 61,	/* Short Preamble */
+	IEEE80211_PARAM_DUMPREGS		= 62,	/* Pretty printed dump of Atheros hardware registers */
+	IEEE80211_PARAM_DOTH_ALGORITHM		= 63,	/* spectrum management algorithm */
+	IEEE80211_PARAM_DOTH_MINCOM   		= 64,	/* minimum number of common channels */
+	IEEE80211_PARAM_DOTH_SLCG		= 65,	/* permil of Stations Lost per Channel Gained */
+	IEEE80211_PARAM_DOTH_SLDG		= 66,	/* permil of Stations Lost per rssi Db Gained */
+	IEEE80211_PARAM_TXCONT			= 67,	/* continuous transmit mode (boolean) */
+	IEEE80211_PARAM_TXCONT_RATE		= 68,	/* continuous transmit mode data rate (in mbit/sec) - will use closest match from current rate table */
+	IEEE80211_PARAM_TXCONT_POWER		= 69,	/* power level in units of 0.5dBm */
+	IEEE80211_PARAM_DFS_TESTMODE		= 70,	/* do not perform DFS actions (i.e. markng DFS and channel change on interference), just report them via debug. */
+	IEEE80211_PARAM_DFS_CHANCHECKTIME	= 71,	/* how long do we wait for chan availability
+							   scans ?
+							   FCC requires 60s, so that is the default. */
+	IEEE80211_PARAM_DFS_NONOCCUPANCYPERIOD	= 72,	/* DFS no-occupancy limit - how long do we stay
+							   off a channel once radar is detected?
+							   FCC requires 30m, so that is the default. */
 };
 
 #define	SIOCG80211STATS			(SIOCDEVPRIVATE+2)
@@ -633,7 +668,7 @@ enum {
 struct ieee80211req_getset_appiebuf {
 	u_int32_t	app_frmtype;		/* management frame type for which buffer is added */
 	u_int32_t	app_buflen;		/* application-supplied buffer length */
-	u_int8_t	app_buf[];		/* application-supplied IE(s) */
+	u_int8_t	app_buf[0];		/* application-supplied IE(s) */
 };
 
 /* Flags ORed by application to set filter for receiving management frames */
