@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_linux.c 1817 2006-11-22 08:34:40Z kelmo $
+ * $Id: ieee80211_linux.c 2915 2007-11-22 02:08:35Z mentor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -60,6 +60,54 @@
 #include <net80211/ieee80211_var.h>
 #include <net80211/ieee80211_monitor.h>
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
+#include <linux/device.h>
+
+/* madwifi_name_type - device name type:
+ * values:	0:	automatically assigned
+ * 		1:	administratively assigned
+ * 		else:	reserved			*/
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+static ssize_t show_madwifi_name_type(struct device *dev,
+		struct device_attribute *attr, char *buf)
+#else
+static ssize_t show_madwifi_name_type(struct class_device *cdev,
+		char *buf)
+#endif
+{
+	ssize_t len = 0;
+
+	len = snprintf(buf, PAGE_SIZE, "1");
+
+	return len;
+}
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+static DEVICE_ATTR(madwifi_name_type, S_IRUGO, show_madwifi_name_type, NULL);
+#else
+static CLASS_DEVICE_ATTR(madwifi_name_type, S_IRUGO, show_madwifi_name_type, NULL);
+#endif
+
+static struct attribute *ieee80211_sysfs_attrs[] = {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+	&dev_attr_madwifi_name_type.attr,
+#else
+	&class_device_attr_madwifi_name_type.attr,
+#endif
+	NULL
+};
+
+static struct attribute_group ieee80211_attr_grp = {
+	.name	= NULL,	/* No seperate (sub-)directory */
+	.attrs	= ieee80211_sysfs_attrs
+};
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17) */
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#define proc_net init_net.proc_net
+#endif
+
 /*
  * Print a console message with the device name prepended.
  */
@@ -86,44 +134,42 @@ if_printf(struct net_device *dev, const char *fmt, ...)
  * can use this interface too.
  */
 struct sk_buff *
+#ifdef IEEE80211_DEBUG_REFCNT
+ieee80211_getmgtframe_debug(u_int8_t **frm, u_int pktlen, 
+		const char* func, int line)
+#else
 ieee80211_getmgtframe(u_int8_t **frm, u_int pktlen)
+#endif
 {
 	const u_int align = sizeof(u_int32_t);
-	struct ieee80211_cb *cb;
 	struct sk_buff *skb;
 	u_int len;
 
 	len = roundup(sizeof(struct ieee80211_frame) + pktlen, 4);
-	skb = dev_alloc_skb(len + align-1);
+#ifdef IEEE80211_DEBUG_REFCNT
+	skb = ieee80211_dev_alloc_skb_debug(len + align - 1, func, line);
+#else
+	skb = ieee80211_dev_alloc_skb(len + align - 1);
+#endif
 	if (skb != NULL) {
 		u_int off = ((unsigned long) skb->data) % align;
 		if (off != 0)
 			skb_reserve(skb, align - off);
 
-		cb = (struct ieee80211_cb *)skb->cb;
-		cb->ni = NULL;
-		cb->flags = 0;
-		cb->next = NULL;
+		SKB_CB(skb)->ni = NULL;
+		SKB_CB(skb)->flags = 0;
+		SKB_CB(skb)->next = NULL;
 
 		skb_reserve(skb, sizeof(struct ieee80211_frame));
 		*frm = skb_put(skb, pktlen);
 	}
 	return skb;
 }
-
-#if 0
-/*
- * Drain a queue of sk_buff's.
- */
-void
-__skb_queue_drain(struct sk_buff_head *q)
-{
-	struct sk_buff *skb;
-
-	while ((skb = __skb_dequeue(q)) != NULL)
-		dev_kfree_skb(skb);
-}
-#endif
+#ifdef IEEE80211_DEBUG_REFCNT
+EXPORT_SYMBOL(ieee80211_getmgtframe_debug);
+#else
+EXPORT_SYMBOL(ieee80211_getmgtframe);
+#endif 
 
 #if IEEE80211_VLAN_TAG_USED
 /*
@@ -162,7 +208,7 @@ ieee80211_vlan_kill_vid(struct net_device *dev, unsigned short vid)
 	struct ieee80211vap *vap = dev->priv;
 
 	if (vap->iv_vlgrp != NULL)
-		vap->iv_vlgrp->vlan_devices[vid] = NULL;
+		vlan_group_set_device(vap->iv_vlgrp, vid, NULL);
 }
 #endif /* IEEE80211_VLAN_TAG_USED */
 
@@ -192,8 +238,8 @@ ieee80211_notify_node_join(struct ieee80211_node *ni, int newassoc)
 	struct net_device *dev = vap->iv_dev;
 	union iwreq_data wreq;
 
-	if (ni == vap->iv_bss) { 
-		if (newassoc) 
+	if (ni == vap->iv_bss) {
+		if (newassoc)
 			netif_carrier_on(dev);
 		memset(&wreq, 0, sizeof(wreq));
 		IEEE80211_ADDR_COPY(wreq.addr.sa_data, ni->ni_bssid);
@@ -246,10 +292,11 @@ ieee80211_notify_sta_stats(struct ieee80211_node *ni)
 	char buf[1024];
 
 	snprintf(buf, sizeof(buf), "%s\nmac=%s\nrx_packets=%u\nrx_bytes=%llu\n"
-			"tx_packets=%u\ntx_bytes=%llu\n", tag, 
-			ether_sprintf(ni->ni_macaddr), ni->ni_stats.ns_rx_data, 
-			ni->ni_stats.ns_rx_bytes, ni->ni_stats.ns_tx_data, 
-			ni->ni_stats.ns_tx_bytes);
+			"tx_packets=%u\ntx_bytes=%llu\n", tag,
+			ether_sprintf(ni->ni_macaddr), ni->ni_stats.ns_rx_data,
+			(unsigned long long)ni->ni_stats.ns_rx_bytes,
+			ni->ni_stats.ns_tx_data,
+			(unsigned long long)ni->ni_stats.ns_tx_bytes);
 	memset(&wreq, 0, sizeof(wreq));
 	wreq.data.length = strlen(buf);
 	wireless_send_event(dev, IWEVCUSTOM, &wreq, buf);
@@ -281,7 +328,8 @@ ieee80211_notify_replay_failure(struct ieee80211vap *vap,
 
 	IEEE80211_NOTE_MAC(vap, IEEE80211_MSG_CRYPTO, wh->i_addr2,
 		"%s replay detected <keyix %d, rsc %llu >",
-		k->wk_cipher->ic_name, k->wk_keyix, rsc );
+		k->wk_cipher->ic_name, k->wk_keyix,
+		(unsigned long long)rsc );
 
 	/* TODO: needed parameters: count, keyid, key type, src address, TSC */
 	snprintf(buf, sizeof(buf), "%s(keyid=%d %scast addr=%s)", tag,
@@ -296,7 +344,7 @@ EXPORT_SYMBOL(ieee80211_notify_replay_failure);
 
 void
 ieee80211_notify_michael_failure(struct ieee80211vap *vap,
-	const struct ieee80211_frame *wh, u_int keyix)
+	const struct ieee80211_frame *wh, ieee80211_keyix_t keyix)
 {
 	static const char *tag = "MLME-MICHAELMICFAILURE.indication";
 	struct net_device *dev = vap->iv_dev;
@@ -309,36 +357,42 @@ ieee80211_notify_michael_failure(struct ieee80211vap *vap,
 
 	/* TODO: needed parameters: count, keyid, key type, src address, TSC */
 	snprintf(buf, sizeof(buf), "%s(keyid=%d %scast addr=%s)", tag,
-		keyix, IEEE80211_IS_MULTICAST(wh->i_addr1) ?  "broad" : "uni",
-		ether_sprintf(wh->i_addr1));
+		keyix, IEEE80211_IS_MULTICAST(wh->i_addr2) ?  "broad" : "uni",
+		ether_sprintf(wh->i_addr2));
 	memset(&wrqu, 0, sizeof(wrqu));
 	wrqu.data.length = strlen(buf);
 	wireless_send_event(dev, IWEVCUSTOM, &wrqu, buf);
 }
 EXPORT_SYMBOL(ieee80211_notify_michael_failure);
 
+/* This function might sleep. Therefore: 
+ * Context: process
+ *
+ * Note that a successful call to this function does not guarantee that
+ * the services provided by the requested module are available:
+ *
+ * "Note that a successful module load does not mean the module did not
+ * then unload and exit on an error of its own. Callers must check that
+ * the service they requested is now available not blindly invoke it."
+ * http://kernelnewbies.org/documents/kdoc/kernel-api/r7338.html
+ */
 int
 ieee80211_load_module(const char *modname)
 {
+#ifdef CONFIG_KMOD
 	int rv;
-	
 	rv = request_module(modname);
-	
 	if (rv < 0)
-		printk(KERN_ERR "couldn't load module '%s' (%d)\n",
-			modname, rv);
-	/*
-	 * XXX Further checking needed if module has been loaded successfully:
-	 *
-	 * "Note that a successful module load does not mean the module did not
-	 * then unload and exit on an error of its own. Callers must check that
-	 * the service they requested is now available not blindly invoke it."
-	 * http://kernelnewbies.org/documents/kdoc/kernel-api/r7338.html
-	 */
+		printk(KERN_ERR "failed to automatically load module: %s; " \
+			"errno: %d\n", modname, rv);
 	return rv;
+#else /* CONFIG_KMOD */
+	printk(KERN_ERR "Unable to load needed module: %s; no support for " \
+			"automatic module loading\n", modname);
+	return -ENOSYS;
+#endif /* CONFIG_KMOD */
 }
 
-#ifdef CONFIG_SYSCTL
 
 static struct proc_dir_entry *proc_madwifi;
 static int proc_madwifi_count = 0;
@@ -346,15 +400,15 @@ static int proc_madwifi_count = 0;
 static int
 proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 {
-        char *p = buf;
-        struct ieee80211_node *ni;
-        struct ieee80211_node_table *nt = (struct ieee80211_node_table *) &vap->iv_ic->ic_sta;
+	char *p = buf;
+	struct ieee80211_node *ni;
+	struct ieee80211_node_table *nt = (struct ieee80211_node_table *) &vap->iv_ic->ic_sta;
 
-        //IEEE80211_NODE_LOCK(nt);                                                                               
-        TAILQ_FOREACH(ni, &nt->nt_node, ni_list) {
-                /* Assume each node needs 500 bytes */
-                if (buf + space < p + 500)
-                        break;
+	IEEE80211_NODE_TABLE_LOCK_IRQ(nt);
+	TAILQ_FOREACH(ni, &nt->nt_node, ni_list) {
+		/* Assume each node needs 500 bytes */
+		if (buf + space < p + 500)
+			break;
 
 		if (ni->ni_vap == vap &&
 		    0 != memcmp(vap->iv_myaddr, ni->ni_macaddr, IEEE80211_ADDR_LEN)) {
@@ -362,14 +416,14 @@ proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 			jiffies_to_timespec(jiffies - ni->ni_last_rx, &t);
 			p += sprintf(p, "macaddr: <%s>\n", ether_sprintf(ni->ni_macaddr));
 			p += sprintf(p, " rssi %d\n", ni->ni_rssi);
-			
-			p += sprintf(p, " last_rx %ld.%06ld\n", 
+
+			p += sprintf(p, " last_rx %ld.%06ld\n",
 				     t.tv_sec, t.tv_nsec / 1000);
 
 		}
         }
-        //IEEE80211_NODE_UNLOCK(nt);                                                                             
-        return (p - buf);
+	IEEE80211_NODE_TABLE_UNLOCK_IRQ(nt);
+	return (p - buf);
 }
 
 static ssize_t
@@ -401,7 +455,7 @@ proc_ieee80211_open(struct inode *inode, struct file *file)
 
 	if (!(file->private_data = kmalloc(sizeof(struct proc_ieee80211_priv), GFP_KERNEL)))
 		return -ENOMEM;
-	/* intially allocate both read and write buffers */
+	/* initially allocate both read and write buffers */
 	pv = (struct proc_ieee80211_priv *) file->private_data;
 	memset(pv, 0, sizeof(struct proc_ieee80211_priv));
 	pv->rbuf = vmalloc(MAX_PROC_IEEE80211_SIZE);
@@ -462,10 +516,10 @@ proc_ieee80211_close(struct inode *inode, struct file *file)
 }
 
 static struct file_operations proc_ieee80211_ops = {
-        .read = proc_ieee80211_read,
-        .write = proc_ieee80211_write,
-        .open = proc_ieee80211_open,
-        .release = proc_ieee80211_close,
+	.read = proc_ieee80211_read,
+	.write = proc_ieee80211_write,
+	.open = proc_ieee80211_open,
+	.release = proc_ieee80211_close,
 };
 
 #ifdef IEEE80211_DEBUG
@@ -482,10 +536,13 @@ IEEE80211_SYSCTL_DECL(ieee80211_sysctl_debug, ctl, write, filp, buffer,
 	if (write) {
 		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
 			lenp, ppos);
-		if (ret == 0)
-			vap->iv_debug = val;
+		if (ret == 0) {
+			vap->iv_debug 		= (val & ~IEEE80211_MSG_IC);
+			vap->iv_ic->ic_debug 	= (val &  IEEE80211_MSG_IC);
+		}
 	} else {
-		val = vap->iv_debug;
+		/* VAP specific and 'global' debug flags */
+		val = vap->iv_debug | vap->iv_ic->ic_debug;
 		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
 			lenp, ppos);
 	}
@@ -521,6 +578,7 @@ IEEE80211_SYSCTL_DECL(ieee80211_sysctl_dev_type, ctl, write, filp, buffer,
 	}
 	return ret;
 }
+
 static int
 IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_nods_only, ctl, write, filp, buffer,
 	lenp, ppos)
@@ -543,6 +601,7 @@ IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_nods_only, ctl, write, filp, buff
 	}
 	return ret;
 }
+
 static int
 IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_txf_len, ctl, write, filp, buffer,
 	lenp, ppos)
@@ -566,7 +625,51 @@ IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_txf_len, ctl, write, filp, buffer
 	return ret;
 }
 
-#define	CTL_AUTO	-2	/* cannot be CTL_ANY or CTL_NONE */
+static int
+IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_phy_errors, ctl, write, filp, buffer,
+	lenp, ppos)
+{
+	struct ieee80211vap *vap = ctl->extra1;
+	u_int val;
+	int ret;
+
+	ctl->data = &val;
+	ctl->maxlen = sizeof(val);
+	if (write) {
+		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
+			lenp, ppos);
+		if (ret == 0)
+			vap->iv_monitor_phy_errors = val;
+	} else {
+		val = vap->iv_monitor_phy_errors;
+		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
+			lenp, ppos);
+	}
+	return ret;
+}
+
+static int
+IEEE80211_SYSCTL_DECL(ieee80211_sysctl_monitor_crc_errors, ctl, write, filp, buffer,
+	lenp, ppos)
+{
+	struct ieee80211vap *vap = ctl->extra1;
+	u_int val;
+	int ret;
+
+	ctl->data = &val;
+	ctl->maxlen = sizeof(val);
+	if (write) {
+		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
+			lenp, ppos);
+		if (ret == 0)
+			vap->iv_monitor_crc_errors = val;
+	} else {
+		val = vap->iv_monitor_crc_errors;
+		ret = IEEE80211_SYSCTL_PROC_DOINTVEC(ctl, write, filp, buffer,
+			lenp, ppos);
+	}
+	return ret;
+}
 
 static const ctl_table ieee80211_sysctl_template[] = {
 #ifdef IEEE80211_DEBUG
@@ -591,6 +694,16 @@ static const ctl_table ieee80211_sysctl_template[] = {
 	  .mode		= 0644,
 	  .proc_handler	= ieee80211_sysctl_monitor_txf_len
 	},
+	{ .ctl_name	= CTL_AUTO,
+	  .procname	= "monitor_phy_errors",
+	  .mode		= 0644,
+	  .proc_handler = ieee80211_sysctl_monitor_phy_errors
+	},
+	{ .ctl_name	= CTL_AUTO,
+	  .procname	= "monitor_crc_errors",
+	  .mode		= 0644,
+	  .proc_handler = ieee80211_sysctl_monitor_crc_errors
+	},
 	/* NB: must be last entry before NULL */
 	{ .ctl_name	= CTL_AUTO,
 	  .procname	= "%parent",
@@ -602,12 +715,31 @@ static const ctl_table ieee80211_sysctl_template[] = {
 };
 
 void
-ieee80211_sysctl_vattach(struct ieee80211vap *vap)
+ieee80211_virtfs_latevattach(struct ieee80211vap *vap)
 {
 	int i, space;
 	char *devname = NULL;
-	struct ieee80211_proc_entry *tmp=NULL;
-	
+	struct ieee80211_proc_entry *tmp = NULL;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
+	int ret;
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+	ret = sysfs_create_group(&vap->iv_dev->dev.kobj, &ieee80211_attr_grp);
+#else
+	ret = sysfs_create_group(&vap->iv_dev->class_dev.kobj, &ieee80211_attr_grp);
+#endif
+	if (ret) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+		sysfs_remove_group(&vap->iv_dev->dev.kobj, &ieee80211_attr_grp);
+#else
+		sysfs_remove_group(&vap->iv_dev->class_dev.kobj, &ieee80211_attr_grp);
+#endif
+		printk("%s: %s - unable to create sysfs attribute group\n", 
+				__func__, vap->iv_dev->name);
+		return;
+	}
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17) */
+
 	space = 5 * sizeof(struct ctl_table) + sizeof(ieee80211_sysctl_template);
 	vap->iv_sysctls = kmalloc(space, GFP_KERNEL);
 	if (vap->iv_sysctls == NULL) {
@@ -643,7 +775,7 @@ ieee80211_sysctl_vattach(struct ieee80211vap *vap)
 		sizeof(ieee80211_sysctl_template));
 
 	/* add in dynamic data references */
-	for (i = 4; vap->iv_sysctls[i].ctl_name; i++)
+	for (i = 4; vap->iv_sysctls[i].procname; i++)
 		if (vap->iv_sysctls[i].extra1 == NULL)
 			vap->iv_sysctls[i].extra1 = vap;
 
@@ -651,9 +783,10 @@ ieee80211_sysctl_vattach(struct ieee80211vap *vap)
 	vap->iv_sysctls[i-1].data = vap->iv_ic->ic_dev->name;	/* XXX? */
 
 	/* and register everything */
-	vap->iv_sysctl_header = register_sysctl_table(vap->iv_sysctls, 1);
+	vap->iv_sysctl_header = ATH_REGISTER_SYSCTL_TABLE(vap->iv_sysctls);
 	if (!vap->iv_sysctl_header) {
 		printk("%s: failed to register sysctls!\n", vap->iv_dev->name);
+		kfree(devname);
 		kfree(vap->iv_sysctls);
 		vap->iv_sysctls = NULL;
 	}
@@ -670,16 +803,16 @@ ieee80211_sysctl_vattach(struct ieee80211vap *vap)
 		proc_madwifi_count++;
 		vap->iv_proc = proc_mkdir(vap->iv_dev->name, proc_madwifi);
 	}
-	
+
 	/* Create a proc entry listing the associated stations */
 	ieee80211_proc_vcreate(vap, &proc_ieee80211_ops, "associated_sta");
 
 	/* Recreate any other proc entries that have been registered */
-		if (vap->iv_proc) {
+	if (vap->iv_proc) {
 		tmp = vap->iv_proc_entries;
 		while (tmp) {
 			if (!tmp->entry) {
-				tmp->entry = create_proc_entry(tmp->name, 
+				tmp->entry = create_proc_entry(tmp->name,
 				PROC_IEEE80211_PERM, vap->iv_proc);
 				tmp->entry->data = vap;
 				tmp->entry->proc_fops = tmp->fileops;
@@ -690,10 +823,10 @@ ieee80211_sysctl_vattach(struct ieee80211vap *vap)
 }
 
 /* Frees all memory used for the list of proc entries */
-void 
+void
 ieee80211_proc_cleanup(struct ieee80211vap *vap)
 {
-	struct ieee80211_proc_entry *tmp=vap->iv_proc_entries;
+	struct ieee80211_proc_entry *tmp = vap->iv_proc_entries;
 	struct ieee80211_proc_entry *next = NULL;
 	while (tmp) {
 		next = tmp->next;
@@ -703,12 +836,12 @@ ieee80211_proc_cleanup(struct ieee80211vap *vap)
 }
 
 /* Called by other modules to register a proc entry under the vap directory */
-int 
-ieee80211_proc_vcreate(struct ieee80211vap *vap, 
+int
+ieee80211_proc_vcreate(struct ieee80211vap *vap,
 		struct file_operations *fileops, char *name)
 {
 	struct ieee80211_proc_entry *entry;
-	struct ieee80211_proc_entry *tmp=NULL;
+	struct ieee80211_proc_entry *tmp = NULL;
 
 	/* Ignore if already in the list */
 	if (vap->iv_proc_entries) {
@@ -723,11 +856,11 @@ ieee80211_proc_vcreate(struct ieee80211vap *vap,
 			tmp = tmp->next;
 		} while (1);
 	}
-	
+
 	/* Create an item in our list for the new entry */
 	entry = kmalloc(sizeof(struct ieee80211_proc_entry), GFP_KERNEL);
 	if (entry == NULL) {
-		printk("%s: no memory for new proc entry (%s)!\n", __func__, 
+		printk("%s: no memory for new proc entry (%s)!\n", __func__,
 				name);
 		return -1;
 	}
@@ -741,7 +874,7 @@ ieee80211_proc_vcreate(struct ieee80211vap *vap,
 		fileops->read = proc_ieee80211_read;
 	if (!fileops->write)
 		fileops->write = proc_ieee80211_write;
-	
+
 	/* Create the entry record */
 	entry->name = name;
 	entry->fileops = fileops;
@@ -750,7 +883,7 @@ ieee80211_proc_vcreate(struct ieee80211vap *vap,
 
 	/* Create the actual proc entry */
 	if (vap->iv_proc) {
-		entry->entry = create_proc_entry(entry->name, 
+		entry->entry = create_proc_entry(entry->name,
 				PROC_IEEE80211_PERM, vap->iv_proc);
 		entry->entry->data = vap;
 		entry->entry->proc_fops = entry->fileops;
@@ -770,10 +903,16 @@ ieee80211_proc_vcreate(struct ieee80211vap *vap,
 EXPORT_SYMBOL(ieee80211_proc_vcreate);
 
 void
-ieee80211_sysctl_vdetach(struct ieee80211vap *vap)
+ieee80211_virtfs_vdetach(struct ieee80211vap *vap)
 {
 	struct ieee80211_proc_entry *tmp=NULL;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
+	sysfs_remove_group(&vap->iv_dev->dev.kobj, &ieee80211_attr_grp);
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
+	sysfs_remove_group(&vap->iv_dev->class_dev.kobj, &ieee80211_attr_grp);
+#endif
+
 	if (vap->iv_sysctl_header) {
 		unregister_sysctl_table(vap->iv_sysctl_header);
 		vap->iv_sysctl_header = NULL;
@@ -797,17 +936,16 @@ ieee80211_sysctl_vdetach(struct ieee80211vap *vap)
 		proc_madwifi_count--;
 	}
 
-	if (vap->iv_sysctls[2].procname) {
+	if (vap->iv_sysctls && vap->iv_sysctls[2].procname) {
 		kfree(vap->iv_sysctls[2].procname);
 		vap->iv_sysctls[2].procname = NULL;
 	}
-	
+
 	if (vap->iv_sysctls) {
 		kfree(vap->iv_sysctls);
 		vap->iv_sysctls = NULL;
 	}
 }
-#endif /* CONFIG_SYSCTL */
 
 /*
  * Format an Ethernet MAC for printing.
@@ -832,25 +970,23 @@ static int
 ieee80211_rcv_dev_event(struct notifier_block *this, unsigned long event,
 	void *ptr)
 {
-#ifdef CONFIG_SYSCTL
 	struct net_device *dev = (struct net_device *) ptr;
 	if (!dev || dev->open != &ieee80211_open)
 		return 0;
 
-        switch (event) {
-        case NETDEV_CHANGENAME:
-		ieee80211_sysctl_vdetach(dev->priv);
-		ieee80211_sysctl_vattach(dev->priv);
+	switch (event) {
+	case NETDEV_CHANGENAME:
+		ieee80211_virtfs_vdetach(dev->priv);
+		ieee80211_virtfs_latevattach(dev->priv);
 		return NOTIFY_DONE;
 	default:
 		break;
-        }
-#endif /* CONFIG_SYSCTL */
-        return 0;
+	}
+	return 0;
 }
 
 static struct notifier_block ieee80211_event_block = {
-        .notifier_call = ieee80211_rcv_dev_event
+	.notifier_call = ieee80211_rcv_dev_event
 };
 
 /*
