@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_proto.c 1832 2006-12-06 05:05:18Z proski $
+ * $Id: ieee80211_proto.c 2938 2007-11-23 23:42:38Z mtaylor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -286,11 +286,12 @@ ieee80211_aclator_get(const char *name)
 }
 EXPORT_SYMBOL(ieee80211_aclator_get);
 
+#ifdef IEEE80211_DEBUG
 void
 ieee80211_print_essid(const u_int8_t *essid, int len)
 {
 	int i;
-	const u_int8_t *p; 
+	const u_int8_t *p;
 
 	if (len > IEEE80211_NWID_LEN)
 		len = IEEE80211_NWID_LEN;
@@ -357,7 +358,7 @@ ieee80211_dump_pkt(struct ieee80211com *ic,
 		break;
 	}
 	if (IEEE80211_QOS_HAS_SEQ(wh)) {
-		const struct ieee80211_qosframe *qwh = 
+		const struct ieee80211_qosframe *qwh =
 			(const struct ieee80211_qosframe *)buf;
 		printf(" QoS [TID %u%s]", qwh->i_qos[0] & IEEE80211_QOS_TID,
 			qwh->i_qos[0] & IEEE80211_QOS_ACKPOLICY ? " ACM" : "");
@@ -390,6 +391,7 @@ ieee80211_dump_pkt(struct ieee80211com *ic,
 	}
 }
 EXPORT_SYMBOL(ieee80211_dump_pkt);
+#endif
 
 int
 ieee80211_fix_rate(struct ieee80211_node *ni, int flags)
@@ -575,13 +577,13 @@ ieee80211_iserp_rateset(struct ieee80211com *ic, struct ieee80211_rateset *rs)
 }
 
 static const struct ieee80211_rateset basic11g[IEEE80211_MODE_MAX] = {
-    { 0 },			/* IEEE80211_MODE_AUTO */
-    { 3, { 12, 24, 48 } },	/* IEEE80211_MODE_11A */
-    { 2, { 2, 4 } },		/* IEEE80211_MODE_11B */
-    { 4, { 2, 4, 11, 22 } },	/* IEEE80211_MODE_11G (mixed b/g) */
-    { 0 },			/* IEEE80211_MODE_FH */
-    { 3, { 12, 24, 48 } },	/* IEEE80211_MODE_TURBO_A */
-    { 4, { 2, 4, 11, 22 } },	/* IEEE80211_MODE_TURBO_G (mixed b/g) */
+	{ 0 },			/* IEEE80211_MODE_AUTO */
+	{ 3, { 12, 24, 48 } },	/* IEEE80211_MODE_11A */
+	{ 2, { 2, 4 } },		/* IEEE80211_MODE_11B */
+	{ 4, { 2, 4, 11, 22 } },	/* IEEE80211_MODE_11G (mixed b/g) */
+	{ 0 },			/* IEEE80211_MODE_FH */
+	{ 3, { 12, 24, 48 } },	/* IEEE80211_MODE_TURBO_A */
+	{ 4, { 2, 4, 11, 22 } },	/* IEEE80211_MODE_TURBO_G (mixed b/g) */
 };
 
 /*
@@ -633,9 +635,9 @@ ieee80211_wme_initparams(struct ieee80211vap *vap)
 {
 	struct ieee80211com *ic = vap->iv_ic;
 
-	IEEE80211_LOCK(ic);
+	IEEE80211_LOCK_IRQ(ic);
 	ieee80211_wme_initparams_locked(vap);
-	IEEE80211_UNLOCK(ic);
+	IEEE80211_UNLOCK_IRQ(ic);
 }
 
 void
@@ -643,74 +645,74 @@ ieee80211_wme_initparams_locked(struct ieee80211vap *vap)
 {
 	struct ieee80211com *ic = vap->iv_ic;
 	struct ieee80211_wme_state *wme = &ic->ic_wme;
-	typedef struct phyParamType { 
-		u_int8_t aifsn; 
+	struct phyParamType {
+		u_int8_t aifsn;
 		u_int8_t logcwmin;
-		u_int8_t logcwmax; 
+		u_int8_t logcwmax;
 		u_int16_t txopLimit;
 		u_int8_t acm;
-	} paramType;
+	};
 	enum ieee80211_phymode mode;
 
-	paramType *pPhyParam, *pBssPhyParam;
+	struct phyParamType *pPhyParam, *pBssPhyParam;
 
 	static struct phyParamType phyParamForAC_BE[IEEE80211_MODE_MAX] = {
-	/* IEEE80211_MODE_AUTO  */ { 3, 4,  6,   0, 0 },    
-	/* IEEE80211_MODE_11A   */ { 3, 4,  6,   0, 0 },
-	/* IEEE80211_MODE_11B   */ { 3, 5,  7,   0, 0 },
-	/* IEEE80211_MODE_11G   */ { 3, 4,  6,   0, 0 },
-	/* IEEE80211_MODE_FH    */ { 3, 5,  7,   0, 0 },
-	/* IEEE80211_MODE_TURBO */ { 2, 3,  5,   0, 0 },
-	/* IEEE80211_MODE_TURBO */ { 2, 3,  5,   0, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 3, 4,  6,   0, 0 },
+		/* IEEE80211_MODE_11A   */ { 3, 4,  6,   0, 0 },
+		/* IEEE80211_MODE_11B   */ { 3, 5,  7,   0, 0 },
+		/* IEEE80211_MODE_11G   */ { 3, 4,  6,   0, 0 },
+		/* IEEE80211_MODE_FH    */ { 3, 5,  7,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 3,  5,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 3,  5,   0, 0 }};
 	static struct phyParamType phyParamForAC_BK[IEEE80211_MODE_MAX] = {
-        /* IEEE80211_MODE_AUTO  */ { 7, 4, 10,   0, 0 },
-        /* IEEE80211_MODE_11A   */ { 7, 4, 10,   0, 0 },
-        /* IEEE80211_MODE_11B   */ { 7, 5, 10,   0, 0 },
-        /* IEEE80211_MODE_11G   */ { 7, 4, 10,   0, 0 },
-	/* IEEE80211_MODE_FH    */ { 7, 5, 10,   0, 0 },
-	/* IEEE80211_MODE_TURBO */ { 7, 3, 10,   0, 0 },
-	/* IEEE80211_MODE_TURBO */ { 7, 3, 10,   0, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 7, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_11A   */ { 7, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_11B   */ { 7, 5, 10,   0, 0 },
+		/* IEEE80211_MODE_11G   */ { 7, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_FH    */ { 7, 5, 10,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 7, 3, 10,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 7, 3, 10,   0, 0 }};
 	static struct phyParamType phyParamForAC_VI[IEEE80211_MODE_MAX] = {
-	/* IEEE80211_MODE_AUTO  */ { 1, 3,  4,  94, 0 },
-	/* IEEE80211_MODE_11A   */ { 1, 3,  4,  94, 0 },
-	/* IEEE80211_MODE_11B   */ { 1, 4,  5, 188, 0 },
-	/* IEEE80211_MODE_11G   */ { 1, 3,  4,  94, 0 },
-	/* IEEE80211_MODE_FH    */ { 1, 4,  5, 188, 0 },
-	/* IEEE80211_MODE_TURBO */ { 1, 2,  3,  94, 0 },
-	/* IEEE80211_MODE_TURBO */ { 1, 2,  3,  94, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 1, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_11A   */ { 1, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_11B   */ { 1, 4,  5, 188, 0 },
+		/* IEEE80211_MODE_11G   */ { 1, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_FH    */ { 1, 4,  5, 188, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  3,  94, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  3,  94, 0 }};
 	static struct phyParamType phyParamForAC_VO[IEEE80211_MODE_MAX] = {
-        /* IEEE80211_MODE_AUTO  */ { 1, 2,  3,  47, 0 },
-	/* IEEE80211_MODE_11A   */ { 1, 2,  3,  47, 0 },
-        /* IEEE80211_MODE_11B   */ { 1, 3,  4, 102, 0 },
-        /* IEEE80211_MODE_11G   */ { 1, 2,  3,  47, 0 },
-	/* IEEE80211_MODE_FH    */ { 1, 3,  4, 102, 0 },
-        /* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 },
-        /* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 1, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_11A   */ { 1, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_11B   */ { 1, 3,  4, 102, 0 },
+		/* IEEE80211_MODE_11G   */ { 1, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_FH    */ { 1, 3,  4, 102, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 }};
 
 	static struct phyParamType bssPhyParamForAC_BE[IEEE80211_MODE_MAX] = {
-        /* IEEE80211_MODE_AUTO  */ { 3, 4, 10,   0, 0 },
-        /* IEEE80211_MODE_11A   */ { 3, 4, 10,   0, 0 },
-        /* IEEE80211_MODE_11B   */ { 3, 5, 10,   0, 0 },
-        /* IEEE80211_MODE_11G   */ { 3, 4, 10,   0, 0 },
-        /* IEEE80211_MODE_FH    */ { 3, 5, 10,   0, 0 },
-        /* IEEE80211_MODE_TURBO */ { 2, 3, 10,   0, 0 },
-        /* IEEE80211_MODE_TURBO */ { 2, 3, 10,   0, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 3, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_11A   */ { 3, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_11B   */ { 3, 5, 10,   0, 0 },
+		/* IEEE80211_MODE_11G   */ { 3, 4, 10,   0, 0 },
+		/* IEEE80211_MODE_FH    */ { 3, 5, 10,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 3, 10,   0, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 3, 10,   0, 0 }};
 	static struct phyParamType bssPhyParamForAC_VI[IEEE80211_MODE_MAX] = {
-        /* IEEE80211_MODE_AUTO  */ { 2, 3,  4,  94, 0 },
-        /* IEEE80211_MODE_11A   */ { 2, 3,  4,  94, 0 },
-        /* IEEE80211_MODE_11B   */ { 2, 4,  5, 188, 0 },
-        /* IEEE80211_MODE_11G   */ { 2, 3,  4,  94, 0 },
-        /* IEEE80211_MODE_FH    */ { 2, 4,  5, 188, 0 },
-        /* IEEE80211_MODE_TURBO */ { 2, 2,  3,  94, 0 },
-        /* IEEE80211_MODE_TURBO */ { 2, 2,  3,  94, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 2, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_11A   */ { 2, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_11B   */ { 2, 4,  5, 188, 0 },
+		/* IEEE80211_MODE_11G   */ { 2, 3,  4,  94, 0 },
+		/* IEEE80211_MODE_FH    */ { 2, 4,  5, 188, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 2,  3,  94, 0 },
+		/* IEEE80211_MODE_TURBO */ { 2, 2,  3,  94, 0 }};
 	static struct phyParamType bssPhyParamForAC_VO[IEEE80211_MODE_MAX] = {
-        /* IEEE80211_MODE_AUTO  */ { 2, 2,  3,  47, 0 },    
-        /* IEEE80211_MODE_11A   */ { 2, 2,  3,  47, 0 },
-        /* IEEE80211_MODE_11B   */ { 2, 3,  4, 102, 0 },
-        /* IEEE80211_MODE_11G   */ { 2, 2,  3,  47, 0 },
-        /* IEEE80211_MODE_FH    */ { 2, 3,  4, 102, 0 },
-	/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 },
-	/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 }};
+		/* IEEE80211_MODE_AUTO  */ { 2, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_11A   */ { 2, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_11B   */ { 2, 3,  4, 102, 0 },
+		/* IEEE80211_MODE_11G   */ { 2, 2,  3,  47, 0 },
+		/* IEEE80211_MODE_FH    */ { 2, 3,  4, 102, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 },
+		/* IEEE80211_MODE_TURBO */ { 1, 2,  2,  47, 0 }};
 
 	int i;
 
@@ -744,11 +746,11 @@ ieee80211_wme_initparams_locked(struct ieee80211vap *vap)
 			break;
 		case WME_AC_BE:
 		default:
-			pPhyParam = &phyParamForAC_BE[mode];		
+			pPhyParam = &phyParamForAC_BE[mode];
 			pBssPhyParam = &bssPhyParamForAC_BE[mode];
 			break;
 		}
-	
+
 		if (ic->ic_opmode == IEEE80211_M_HOSTAP) {
 			wme->wme_wmeChanParams.cap_wmeParams[i].wmep_acm =
 				pPhyParam->acm;
@@ -771,7 +773,7 @@ ieee80211_wme_initparams_locked(struct ieee80211vap *vap)
 				pBssPhyParam->logcwmax;
 			wme->wme_wmeChanParams.cap_wmeParams[i].wmep_txopLimit =
 				pBssPhyParam->txopLimit;
-		}	
+		}
 		wme->wme_wmeBssChanParams.cap_wmeParams[i].wmep_acm =
 			pBssPhyParam->acm;
 		wme->wme_wmeBssChanParams.cap_wmeParams[i].wmep_aifsn =
@@ -786,7 +788,7 @@ ieee80211_wme_initparams_locked(struct ieee80211vap *vap)
 	/* NB: check ic_bss to avoid NULL deref on initial attach */
 	if (vap->iv_bss != NULL) {
 		/*
-		 * Calculate agressive mode switching threshold based
+		 * Calculate aggressive mode switching threshold based
 		 * on beacon interval.
 		 */
 		wme->wme_hipri_switch_thresh =
@@ -803,18 +805,20 @@ ieee80211_wme_updateparams_locked(struct ieee80211vap *vap)
 {
 	static const struct { u_int8_t aifsn; u_int8_t logcwmin; u_int8_t logcwmax; u_int16_t txopLimit;}
 	phyParam[IEEE80211_MODE_MAX] = {
-          /* IEEE80211_MODE_AUTO  */ { 2, 4, 10, 64 },	
-          /* IEEE80211_MODE_11A   */ { 2, 4, 10, 64 },
-          /* IEEE80211_MODE_11B   */ { 2, 5, 10, 64 },
-          /* IEEE80211_MODE_11G   */ { 2, 4, 10, 64 },
-          /* IEEE80211_MODE_FH    */ { 2, 5, 10, 64 },
-          /* IEEE80211_MODE_TURBO */ { 1, 3, 10, 64 }};
+	      /* IEEE80211_MODE_AUTO  */ { 2, 4, 10, 64 },
+	      /* IEEE80211_MODE_11A   */ { 2, 4, 10, 64 },
+	      /* IEEE80211_MODE_11B   */ { 2, 5, 10, 64 },
+	      /* IEEE80211_MODE_11G   */ { 2, 4, 10, 64 },
+	      /* IEEE80211_MODE_FH    */ { 2, 5, 10, 64 },
+	      /* IEEE80211_MODE_TURBO */ { 1, 3, 10, 64 }};
 	struct ieee80211com *ic = vap->iv_ic;
 	struct ieee80211_wme_state *wme = &ic->ic_wme;
 	enum ieee80211_phymode mode;
 	int i;
 
-       	/* set up the channel access parameters for the physical device */
+	IEEE80211_LOCK_ASSERT(vap->iv_ic);
+
+	/* set up the channel access parameters for the physical device */
 
 	for (i = 0; i < WME_NUM_AC; i++) {
 		wme->wme_chanParams.cap_wmeParams[i].wmep_aifsn =
@@ -845,7 +849,7 @@ ieee80211_wme_updateparams_locked(struct ieee80211vap *vap)
 		mode = ieee80211_chan2mode(ic->ic_bsschan);
 	else
 		mode = IEEE80211_MODE_AUTO;
-        if ((vap->iv_opmode == IEEE80211_M_HOSTAP &&
+	    if ((vap->iv_opmode == IEEE80211_M_HOSTAP &&
 	     (wme->wme_flags & WME_F_AGGRMODE) != 0) ||
 	    (vap->iv_opmode != IEEE80211_M_HOSTAP &&
 	     (vap->iv_bss->ni_flags & IEEE80211_NODE_QOS) == 0) ||
@@ -864,7 +868,7 @@ ieee80211_wme_updateparams_locked(struct ieee80211vap *vap)
 		wme->wme_chanParams.cap_wmeParams[WME_AC_BE].wmep_logcwmin =
 			phyParam[mode].logcwmin;
 		wme->wme_chanParams.cap_wmeParams[WME_AC_BE].wmep_logcwmax =
-			phyParam[mode].logcwmax;		
+			phyParam[mode].logcwmax;
 		wme->wme_chanParams.cap_wmeParams[WME_AC_BE].wmep_txopLimit =
 			burstEnabled ? phyParam[mode].txopLimit : 0;
 		wme->wme_bssChanParams.cap_wmeParams[WME_AC_BE].wmep_aifsn =
@@ -874,17 +878,17 @@ ieee80211_wme_updateparams_locked(struct ieee80211vap *vap)
 		wme->wme_bssChanParams.cap_wmeParams[WME_AC_BE].wmep_logcwmax =
 			phyParam[mode].logcwmax;
 		wme->wme_bssChanParams.cap_wmeParams[WME_AC_BE].wmep_txopLimit =
-			burstEnabled ? phyParam[mode].txopLimit : 0;		
+			burstEnabled ? phyParam[mode].txopLimit : 0;
 	}
-	
+
 	if (ic->ic_opmode == IEEE80211_M_HOSTAP &&
 	    ic->ic_sta_assoc < 2 && (wme->wme_flags & WME_F_AGGRMODE) != 0) {
 		static const u_int8_t logCwMin[IEEE80211_MODE_MAX] = {
-			/* IEEE80211_MODE_AUTO  */   3,		
+			/* IEEE80211_MODE_AUTO  */   3,
 			/* IEEE80211_MODE_11A   */   3,
 			/* IEEE80211_MODE_11B   */   4,
 			/* IEEE80211_MODE_11G   */   3,
-			/* IEEE80211_MODE_FH    */   4,	
+			/* IEEE80211_MODE_FH    */   4,
 			/* IEEE80211_MODE_TURBO_A */ 3,
 			/* IEEE80211_MODE_TURBO_G */ 3
 		};
@@ -912,15 +916,17 @@ ieee80211_wme_updateparams_locked(struct ieee80211vap *vap)
 			wme->wme_bssChanParams.cap_info_count);
 }
 
+EXPORT_SYMBOL(ieee80211_wme_updateparams);
+
 void
 ieee80211_wme_updateparams(struct ieee80211vap *vap)
 {
 	struct ieee80211com *ic = vap->iv_ic;
 
 	if (ic->ic_caps & IEEE80211_C_WME) {
-		IEEE80211_LOCK(ic);
+		IEEE80211_LOCK_IRQ(ic);
 		ieee80211_wme_updateparams_locked(vap);
-		IEEE80211_UNLOCK(ic);
+		IEEE80211_UNLOCK_IRQ(ic);
 	}
 }
 
@@ -966,7 +972,7 @@ ieee80211_init(struct net_device *dev, int forcescan)
 			 * we should be able to apply any new state/parameters
 			 * simply by re-associating.  Otherwise we need to
 			 * re-scan to select an appropriate ap.
-			 */ 
+			 */
 			if (vap->iv_state != IEEE80211_S_RUN || forcescan)
 				ieee80211_new_state(vap, IEEE80211_S_SCAN, 0);
 			else
@@ -977,7 +983,7 @@ ieee80211_init(struct net_device *dev, int forcescan)
 			 * be brought to init.
 			 */
 			if (vap->iv_state == IEEE80211_S_RUN)
-				ieee80211_new_state(vap, IEEE80211_S_INIT, -1); 		
+				ieee80211_new_state(vap, IEEE80211_S_INIT, -1);
 			/*
 			 * For monitor+wds modes there's nothing to do but
 			 * start running.  Otherwise, if this is the first
@@ -1003,7 +1009,7 @@ ieee80211_open(struct net_device *dev)
 }
 
 /*
- * Start all runnable vap's on a device.
+ * Start all runnable VAPs on a device.
  */
 void
 ieee80211_start_running(struct ieee80211com *ic)
@@ -1022,7 +1028,7 @@ EXPORT_SYMBOL(ieee80211_start_running);
 
 /*
  * Stop a vap.  We force it down using the state machine
- * then mark it's device not running.  If this is the last
+ * then mark its device not running.  If this is the last
  * vap running on the underlying device then we close it
  * too to ensure it will be properly initialized when the
  * next vap is brought up.
@@ -1060,7 +1066,7 @@ ieee80211_stop(struct net_device *dev)
 EXPORT_SYMBOL(ieee80211_stop);
 
 /*
- * Stop all vap's running on a device.
+ * Stop all VAPs running on a device.
  */
 void
 ieee80211_stop_running(struct ieee80211com *ic)
@@ -1087,8 +1093,10 @@ EXPORT_SYMBOL(ieee80211_stop_running);
 void
 ieee80211_dturbo_switch(struct ieee80211com *ic, int newflags)
 {
+#ifdef IEEE80211_DEBUG
 	/* XXX use first vap for debug flags */
 	struct ieee80211vap *vap = TAILQ_FIRST(&ic->ic_vaps);
+#endif
 	struct ieee80211_channel *chan;
 
 	chan = ieee80211_find_channel(ic, ic->ic_bsschan->ic_freq, newflags);
@@ -1106,7 +1114,6 @@ ieee80211_dturbo_switch(struct ieee80211com *ic, int newflags)
 		chan->ic_freq, chan->ic_flags);
 
 	ic->ic_bsschan = chan;
-	ic->ic_prevchan = ic->ic_curchan;
 	ic->ic_curchan = chan;
 	ic->ic_set_channel(ic);
 	/* NB: do not need to reset ERP state because in sta mode */
@@ -1232,7 +1239,7 @@ ieee80211_new_state(struct ieee80211vap *vap, enum ieee80211_state nstate, int a
 	struct ieee80211com *ic = vap->iv_ic;
 	int rc;
 
-	/* grab the lock so that only one vap can go through transistion at any time */
+	/* grab the lock so that only one vap can go through transition at any time */
 	IEEE80211_VAPS_LOCK_BH(ic);
 	rc = vap->iv_newstate(vap, nstate, arg);
 	IEEE80211_VAPS_UNLOCK_BH(ic);
@@ -1316,7 +1323,7 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 				ieee80211_create_ibss(vap, vap->iv_des_chan);
 			} else {
 				ieee80211_check_scan(vap,
-					IEEE80211_SCAN_ACTIVE | 
+					IEEE80211_SCAN_ACTIVE |
 					IEEE80211_SCAN_FLUSH,
 					IEEE80211_SCAN_FOREVER,
 					vap->iv_des_nssid, vap->iv_des_ssid,
@@ -1451,20 +1458,45 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 				 */
 				ieee80211_create_ibss(vap, ic->ic_curchan);
 
-				/*
-				 * In wds mode allocate and initialize peer node
-				 */				
+				/* In WDS mode, allocate and initialize peer node. */
 				if (vap->iv_opmode == IEEE80211_M_WDS) {
-					struct ieee80211_node *wds_ni;
-					wds_ni = ieee80211_alloc_node(&ic->ic_sta, vap, vap->wds_mac);
+					/* XXX: This is horribly non-atomic. */
+					struct ieee80211_node *wds_ni =
+						ieee80211_find_node(&ic->ic_sta,
+								vap->wds_mac);
+
+					if (wds_ni == NULL) {
+						wds_ni = ieee80211_alloc_node_table(
+								vap,
+								vap->wds_mac);
+						if (wds_ni != NULL)
+							ieee80211_add_wds_addr(
+									&ic->ic_sta,
+									wds_ni,
+									vap->wds_mac,
+									1);
+						else
+							IEEE80211_DPRINTF(
+									vap,
+									IEEE80211_MSG_NODE,
+									"%s: Unable to "
+									"allocate node for "
+									"WDS: %s\n",
+									__func__,
+									ether_sprintf(
+										vap->wds_mac)
+									);
+					}
+
 					if (wds_ni != NULL) {
-						if (ieee80211_add_wds_addr(&ic->ic_sta, wds_ni, vap->wds_mac, 1) == 0) {
-							ieee80211_node_authorize(wds_ni);
-							wds_ni->ni_chan = vap->iv_bss->ni_chan;
-							wds_ni->ni_capinfo = ni->ni_capinfo;
-							wds_ni->ni_associd = 1;
-							wds_ni->ni_ath_flags = vap->iv_ath_cap;
-						}
+						ieee80211_node_authorize(wds_ni);
+						wds_ni->ni_chan =
+							vap->iv_bss->ni_chan;
+						wds_ni->ni_capinfo =
+							ni->ni_capinfo;
+						wds_ni->ni_associd = 1;
+						wds_ni->ni_ath_flags =
+							vap->iv_ath_cap;
 					}
 				}
 				break;
@@ -1497,7 +1529,7 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 			if (vap->iv_opmode == IEEE80211_M_STA) {
 				ieee80211_scan_assoc_success(ic,
 					ni->ni_macaddr);
-				ieee80211_notify_node_join(ni, 
+				ieee80211_notify_node_join(ni,
 					(arg == IEEE80211_FC0_SUBTYPE_ASSOC_RESP) | \
 					(arg == IEEE80211_FC0_SUBTYPE_REASSOC_RESP));
 			}
@@ -1512,10 +1544,10 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 			vap->iv_swbmiss.data = (unsigned long) vap;
 			vap->iv_swbmiss_period = IEEE80211_TU_TO_JIFFIES(
 				IEEE80211_SWBMISS_THRESHOLD * ni->ni_intval);
-			
+
 			mod_timer(&vap->iv_swbmiss, jiffies + vap->iv_swbmiss_period);
 		}
-		
+
 		/*
 		 * Start/stop the authenticator when operating as an
 		 * AP.  We delay until here to allow configuration to
@@ -1543,7 +1575,7 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 		    !(vap->iv_flags & IEEE80211_F_XR)) {
 			vap->iv_xrvapstart.function = ieee80211_start_xrvap;
 			vap->iv_xrvapstart.data = (unsigned long) vap->iv_xrvap;
-			mod_timer(&vap->iv_xrvapstart,jiffies+HZ); /* start xr vap on next second */
+			mod_timer(&vap->iv_xrvapstart, jiffies + HZ); /* start xr vap on next second */
 			/* 
 			 * do not let the normal vap automatically bring up XR vap.
 			 * let the timer handler start the XR vap. if you let the
@@ -1551,7 +1583,7 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 			 * have the bssid initialized and the XR vap will use the
 			 * invalid bssid in XRIE of its beacon.
 			 */
-			if (vap->iv_xrvap->iv_flags_ext & IEEE80211_FEXT_SCAN_PENDING) 
+			if (vap->iv_xrvap->iv_flags_ext & IEEE80211_FEXT_SCAN_PENDING)
 				vap->iv_xrvap->iv_flags_ext &= ~IEEE80211_FEXT_SCAN_PENDING;
 		}
 		/*
@@ -1560,10 +1592,10 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 		 * with the xr vaps MAC address.
 		 */
 		if (vap->iv_flags & IEEE80211_F_XR)
-			vap->iv_xrvap->iv_flags |= IEEE80211_F_XRUPDATE; 
+			vap->iv_xrvap->iv_flags |= IEEE80211_F_XRUPDATE;
 #endif
 		break;
-	}	
+	}
 	return 0;
 }
 
@@ -1625,7 +1657,7 @@ ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int ar
 				vap->iv_flags_ext |= IEEE80211_FEXT_SCAN_PENDING;
 			}
 		} else {
-			/* the VAP is forced to scan, we need to change all other vap's state
+			/* the VAP is forced to scan, we need to change all other VAPs state
 			 * to INIT and pend for the scan completion */
 			TAILQ_FOREACH(tmpvap, &ic->ic_vaps, iv_next) {
 				if (vap != tmpvap && tmpvap->iv_state != IEEE80211_S_INIT) {
@@ -1638,7 +1670,7 @@ ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int ar
 			__ieee80211_newstate(vap, nstate, arg);
 		}
 		break;
-		
+
 	case IEEE80211_S_RUN:
 		if (ostate == IEEE80211_S_SCAN ||		/* AP coming out of scan */
 		    vap->iv_opmode == IEEE80211_M_STA) {	/* STA in WDS/Repeater needs to bring up other VAPs  */
