@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_scan_sta.c 1721 2006-09-20 08:45:13Z mentor $
+ * $Id: ieee80211_scan_sta.c 2736 2007-10-11 15:40:49Z mentor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -64,16 +64,13 @@
 #define	STA_FAILS_AGE	(2 * 60)	/* time before clearing fails (secs) */
 #define	STA_PURGE_SCANS	2		/* age for purging entries (scans) */
 
-/* XXX tunable */
-#define	STA_RSSI_MIN	8		/* min acceptable rssi */
-
 #define RSSI_LPF_LEN	10
 #define	RSSI_EP_MULTIPLIER	(1<<7)	/* pow2 to optimize out * and / */
 #define RSSI_IN(x)		((x) * RSSI_EP_MULTIPLIER)
 #define LPF_RSSI(x, y, len)	(((x) * ((len) - 1) + (y)) / (len))
 #define RSSI_LPF(x, y) do {						\
-    if ((y) >= -20)							\
-    	x = LPF_RSSI((x), RSSI_IN((y)), RSSI_LPF_LEN);			\
+	if ((y) >= -20)							\
+		x = LPF_RSSI((x), RSSI_IN((y)), RSSI_LPF_LEN);			\
 } while (0)
 #define	EP_RND(x, mul) \
 	((((x)%(mul)) >= ((mul)/2)) ? howmany(x, mul) : (x)/(mul))
@@ -110,6 +107,24 @@ struct sta_table {
 	int (*st_action)(struct ieee80211vap *, const struct ieee80211_scan_entry *);
 };
 
+#define	SCAN_STA_LOCK_INIT(_st, _name)					\
+	spin_lock_init(&(_st)->st_lock)
+#define	SCAN_STA_LOCK_DESTROY(_st)
+#define	SCAN_STA_LOCK_IRQ(_st) do {					\
+	unsigned long __stlockflags;					\
+	spin_lock_irqsave(&(_st)->st_lock, __stlockflags);
+#define	SCAN_STA_UNLOCK_IRQ(_st)					\
+	spin_unlock_irqrestore(&(_st)->st_lock, __stlockflags);		\
+} while (0)
+#define	SCAN_STA_UNLOCK_IRQ_EARLY(_st)					\
+	spin_unlock_irqrestore(&(_st)->st_lock, __stlockflags);
+
+#define	SCAN_STA_GEN_LOCK_INIT(_st, _name)				\
+	spin_lock_init(&(_st)->st_scanlock)
+#define	SCAN_STA_GEN_LOCK_DESTROY(_st)
+#define	SCAN_STA_GEN_LOCK(_st)		spin_lock(&(_st)->st_scanlock);
+#define	SCAN_STA_GEN_UNLOCK(_st)	spin_unlock(&(_st)->st_scanlock);
+
 static void sta_flush_table(struct sta_table *);
 static int match_bss(struct ieee80211vap *, const struct ieee80211_scan_state *,
 	const struct sta_entry *);
@@ -129,8 +144,8 @@ sta_attach(struct ieee80211_scan_state *ss)
 		M_80211_SCAN, M_NOWAIT | M_ZERO);
 	if (st == NULL)
 		return 0;
-	spin_lock_init(&st->st_lock);
-	spin_lock_init(&st->st_scanlock);
+	SCAN_STA_LOCK_INIT(st, "scan_sta");
+	SCAN_STA_GEN_LOCK_INIT(st, "scan_sta_gen");
 	TAILQ_INIT(&st->st_entry);
 	IEEE80211_INIT_TQUEUE(&st->st_actiontq, action_tasklet, ss);
 	ss->ss_priv = st;
@@ -163,9 +178,9 @@ sta_flush(struct ieee80211_scan_state *ss)
 {
 	struct sta_table *st = ss->ss_priv;
 
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	sta_flush_table(st);
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 	ss->ss_last = 0;
 	return 0;
 }
@@ -200,7 +215,7 @@ saveie(u_int8_t **iep, const u_int8_t *ie)
  */
 static int
 sta_add(struct ieee80211_scan_state *ss, const struct ieee80211_scanparams *sp,
-	const struct ieee80211_frame *wh, int subtype, int rssi, int rstamp)
+	const struct ieee80211_frame *wh, int subtype, int rssi, u_int64_t rtsf)
 {
 #define	ISPROBE(_st)	((_st) == IEEE80211_FC0_SUBTYPE_PROBE_RESP)
 #define	PICK1ST(_ss) \
@@ -215,39 +230,40 @@ sta_add(struct ieee80211_scan_state *ss, const struct ieee80211_scanparams *sp,
 	int hash;
 
 	hash = STA_HASH(macaddr);
-	spin_lock(&st->st_lock);  
+	SCAN_STA_LOCK_IRQ(st);
 	LIST_FOREACH(se, &st->st_hash[hash], se_hash)
 		if (IEEE80211_ADDR_EQ(se->base.se_macaddr, macaddr) &&
-		    sp->ssid[1] == se->base.se_ssid[1] && 
+		    sp->ssid[1] == se->base.se_ssid[1] &&
 		    !memcmp(se->base.se_ssid+2, sp->ssid+2, se->base.se_ssid[1]))
 			goto found;
 
 	MALLOC(se, struct sta_entry *, sizeof(struct sta_entry),
 		M_80211_SCAN, M_NOWAIT | M_ZERO);
 	if (se == NULL) {
-		spin_unlock(&st->st_lock);
+		SCAN_STA_UNLOCK_IRQ_EARLY(st);
 		return 0;
 	}
 	se->se_scangen = st->st_scangen-1;
 	IEEE80211_ADDR_COPY(se->base.se_macaddr, macaddr);
 	TAILQ_INSERT_TAIL(&st->st_entry, se, se_list);
 	LIST_INSERT_HEAD(&st->st_hash[hash], se, se_hash);
+
 found:
 	ise = &se->base;
+
 	/* XXX ap beaconing multiple ssid w/ same bssid */
 	if (sp->ssid[1] != 0 &&
 	    (ISPROBE(subtype) || ise->se_ssid[1] == 0))
 		memcpy(ise->se_ssid, sp->ssid, 2 + sp->ssid[1]);
-	KASSERT(sp->rates[1] <= IEEE80211_RATE_MAXSIZE,
-		("rate set too large: %u", sp->rates[1]));
-	memcpy(ise->se_rates, sp->rates, 2 + sp->rates[1]);
+
+	memcpy(ise->se_rates, sp->rates, 
+			2 + IEEE80211_SANITISE_RATESIZE(sp->rates[1]));
 	if (sp->xrates != NULL) {
-		/* XXX validate xrates[1] */
-		KASSERT(sp->xrates[1] <= IEEE80211_RATE_MAXSIZE,
-			("xrate set too large: %u", sp->xrates[1]));
-		memcpy(ise->se_xrates, sp->xrates, 2 + sp->xrates[1]);
+		memcpy(ise->se_xrates, sp->xrates, 
+				2 + IEEE80211_SANITISE_RATESIZE(sp->xrates[1]));
 	} else
 		ise->se_xrates[1] = 0;
+
 	IEEE80211_ADDR_COPY(ise->se_bssid, wh->i_addr3);
 	/*
 	 * Record rssi data using extended precision LPF filter.
@@ -257,7 +273,7 @@ found:
 	else					/* avg w/ previous samples */
 		RSSI_LPF(se->se_avgrssi, rssi);
 	se->base.se_rssi = RSSI_GET(se->se_avgrssi);
-	ise->se_rstamp = rstamp;
+	ise->se_rtsf = rtsf;
 	memcpy(ise->se_tstamp.data, sp->tstamp, sizeof(ise->se_tstamp));
 	ise->se_intval = sp->bintval;
 	ise->se_capinfo = sp->capinfo;
@@ -287,7 +303,7 @@ found:
 	se->se_seen = 1;
 	se->se_notseen = 0;
 
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 
 	/*
 	 * If looking for a quick choice and nothing's
@@ -438,7 +454,7 @@ checktable(const struct scanlist *scan, const struct ieee80211_channel *c)
 
 	for (; scan->list != NULL; scan++) {
 		for (i = 0; i < scan->count; i++)
-			if (scan->list[i] == c->ic_freq) 
+			if (scan->list[i] == c->ic_freq)
 				return 1;
 	}
 	return 0;
@@ -457,7 +473,7 @@ sta_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 	enum ieee80211_phymode mode;
 	struct ieee80211_channel *c;
 	int i;
-	
+
 	ss->ss_last = 0;
 	/*
 	 * Use the table of ordered channels to construct the list
@@ -519,10 +535,10 @@ sta_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 			 * If a desired mode was specified, scan only 
 			 * channels that satisfy that constraint.
 			 */
-			if (vap->iv_des_mode != mode) 
+			if (vap->iv_des_mode != mode)
 				continue;
-			
-		} 
+
+		}
 		if (!checktable(staScanTable, c))
 			ss->ss_chans[ss->ss_last++] = c;
 	}
@@ -649,7 +665,7 @@ check_rate(struct ieee80211vap *vap, const struct ieee80211_scan_entry *se)
 	const u_int8_t *rs;
 
 	okrate = badrate = fixedrate = 0;
-	
+
 	if (IEEE80211_IS_CHAN_HALF(se->se_chan))
 		srs = &ic->ic_sup_half_rates;
 	else if (IEEE80211_IS_CHAN_QUARTER(se->se_chan))
@@ -714,8 +730,8 @@ match_bss(struct ieee80211vap *vap,
 {
 	struct ieee80211com *ic = vap->iv_ic;
 	const struct ieee80211_scan_entry *se = &se0->base;
-        u_int8_t rate;
-        int fail;
+	u_int8_t rate;
+	int fail;
 
 	fail = 0;
 	if (isclr(ic->ic_chan_active, ieee80211_chan2ieee(ic, se->se_chan)))
@@ -759,10 +775,8 @@ match_bss(struct ieee80211vap *vap,
 		fail |= 0x40;
 	if (se0->se_notseen >= STA_PURGE_SCANS)
 		fail |= 0x80;
-	if (se->se_rssi < STA_RSSI_MIN)
-		fail |= 0x100;
 #ifdef IEEE80211_DEBUG
-	if (ieee80211_msg(vap, IEEE80211_MSG_SCAN | IEEE80211_MSG_ROAM)) {
+	if (ieee80211_msg_is_reported(vap, IEEE80211_MSG_SCAN | IEEE80211_MSG_ROAM)) {
 		printf(" %03x", fail);
 		printf(" %c %s",
 			fail & 0x40 ? '=' : fail & 0x80 ? '^' : fail ? '-' : '+',
@@ -771,7 +785,7 @@ match_bss(struct ieee80211vap *vap,
 			fail & 0x20 ? '!' : ' ');
 		printf(" %3d%c", ieee80211_chan2ieee(ic, se->se_chan),
 			fail & 0x01 ? '!' : ' ');
-		printf(" %+4d%c", se->se_rssi, fail & 0x100 ? '!' : ' ');
+		printf(" %+4d", se->se_rssi);
 		printf(" %2dM%c", (rate & IEEE80211_RATE_VAL) / 2,
 			fail & 0x08 ? '!' : ' ');
 		printf(" %4s%c",
@@ -792,9 +806,8 @@ static void
 sta_update_notseen(struct sta_table *st)
 {
 	struct sta_entry *se;
-	unsigned long stlockflags;
 
-	spin_lock_irqsave(&st->st_lock, stlockflags);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH(se, &st->st_entry, se_list) {
 		/*
 		 * If seen then reset and don't bump the count;
@@ -808,20 +821,19 @@ sta_update_notseen(struct sta_table *st)
 		else
 			se->se_notseen++;
 	}
-	spin_unlock_irqrestore(&st->st_lock, stlockflags);
+	SCAN_STA_UNLOCK_IRQ(st);
 }
 
 static void
 sta_dec_fails(struct sta_table *st)
 {
 	struct sta_entry *se;
-	unsigned long stlockflags;
 
-	spin_lock_irqsave(&st->st_lock, stlockflags);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH(se, &st->st_entry, se_list)
 		if (se->se_fails)
 			se->se_fails--;
-	spin_unlock_irqrestore(&st->st_lock, stlockflags);
+	SCAN_STA_UNLOCK_IRQ(st);
 }
 
 static struct sta_entry *
@@ -829,11 +841,10 @@ select_bss(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 {
 	struct sta_table *st = ss->ss_priv;
 	struct sta_entry *se, *selbs = NULL;
-	unsigned long stlockflags;
 
 	IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN | IEEE80211_MSG_ROAM, " %s\n",
 		"macaddr          bssid         chan  rssi  rate flag  wep  essid");
-	spin_lock_irqsave(&st->st_lock, stlockflags);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH(se, &st->st_entry, se_list) {
 		if (match_bss(vap, ss, se) == 0) {
 			if (selbs == NULL)
@@ -842,7 +853,7 @@ select_bss(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 				selbs = se;
 		}
 	}
-	spin_unlock_irqrestore(&st->st_lock, stlockflags);
+	SCAN_STA_UNLOCK_IRQ(st);
 
 	return selbs;
 }
@@ -891,7 +902,7 @@ notfound:
 		 * If nothing suitable was found decrement
 		 * the failure counts so entries will be
 		 * reconsidered the next time around.  We
-		 * really want to do this only for sta's
+		 * really want to do this only for STAs
 		 * where we've previously had some success.
 		 */
 		sta_dec_fails(st);
@@ -930,11 +941,11 @@ sta_lookup(struct sta_table *st, const u_int8_t macaddr[IEEE80211_ADDR_LEN])
 	struct sta_entry *se;
 	int hash = STA_HASH(macaddr);
 
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	LIST_FOREACH(se, &st->st_hash[hash], se_hash)
 		if (IEEE80211_ADDR_EQ(se->base.se_macaddr, macaddr))
 			break;
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 
 	return se;		/* NB: unlocked */
 }
@@ -1024,7 +1035,7 @@ sta_age(struct ieee80211_scan_state *ss)
 	struct sta_table *st = ss->ss_priv;
 	struct sta_entry *se, *next;
 
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH_SAFE(se, &st->st_entry, se_list, next) {
 		if (se->se_notseen > STA_PURGE_SCANS) {
 			TAILQ_REMOVE(&st->st_entry, se, se_list);
@@ -1032,7 +1043,7 @@ sta_age(struct ieee80211_scan_state *ss)
 			FREE(se, M_80211_SCAN);
 		}
 	}
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 	/*
 	 * If rate control is enabled check periodically to see if
 	 * we should roam from our current connection to one that
@@ -1055,7 +1066,7 @@ sta_age(struct ieee80211_scan_state *ss)
  * the callback function on each one.
  */
 static int
-sta_iterate(struct ieee80211_scan_state *ss, 
+sta_iterate(struct ieee80211_scan_state *ss,
 	ieee80211_scan_iter_func *f, void *arg)
 {
 	struct sta_table *st = ss->ss_priv;
@@ -1063,30 +1074,31 @@ sta_iterate(struct ieee80211_scan_state *ss,
 	u_int gen;
 	int res = 0;
 
-	spin_lock(&st->st_scanlock);
+	SCAN_STA_GEN_LOCK(st);
 	gen = st->st_scangen++;
 restart:
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH(se, &st->st_entry, se_list) {
 		if (se->se_scangen != gen) {
 			se->se_scangen = gen;
 			/* update public state */
 			se->base.se_age = jiffies - se->se_lastupdate;
-			spin_unlock(&st->st_lock);
+			SCAN_STA_UNLOCK_IRQ_EARLY(st);
+
 			res = (*f)(arg, &se->base);
 
-			if(res != 0) {
+			if (res != 0)
 			  /* We probably ran out of buffer space. */
 			  goto done;
-			}
+
 			goto restart;
 		}
 	}
 
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 
  done:
-	spin_unlock(&st->st_scanlock);
+	SCAN_STA_GEN_UNLOCK(st);
 
 	return res;
 }
@@ -1154,7 +1166,7 @@ adhoc_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 	struct sta_table *st = ss->ss_priv;
 	const struct scanlist *scan;
 	enum ieee80211_phymode mode;
-	
+
 	ss->ss_last = 0;
 	/*
 	 * Use the table of ordered channels to construct the list
@@ -1235,7 +1247,7 @@ adhoc_pick_channel(struct ieee80211_scan_state *ss)
 	bestchan = NULL;
 	bestrssi = -1;
 
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	for (i = 0; i < ss->ss_last; i++) {
 		c = ss->ss_chans[i];
 		maxrssi = 0;
@@ -1248,7 +1260,7 @@ adhoc_pick_channel(struct ieee80211_scan_state *ss)
 		if (bestchan == NULL || maxrssi < bestrssi)
 			bestchan = c;
 	}
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 
 	return bestchan;
 }
@@ -1322,7 +1334,7 @@ adhoc_pick_bss(struct ieee80211_scan_state *ss, struct ieee80211vap *vap,
 		 * If nothing suitable was found decrement
 		 * the failure counts so entries will be
 		 * reconsidered the next time around.  We
-		 * really want to do this only for sta's
+		 * really want to do this only for STAs
 		 * where we've previously had some success.
 		 */
 		sta_dec_fails(st);
@@ -1349,7 +1361,7 @@ adhoc_age(struct ieee80211_scan_state *ss)
 	struct sta_table *st = ss->ss_priv;
 	struct sta_entry *se, *next;
 
-	spin_lock(&st->st_lock);
+	SCAN_STA_LOCK_IRQ(st);
 	TAILQ_FOREACH_SAFE(se, &st->st_entry, se_list, next) {
 		if (se->se_notseen > STA_PURGE_SCANS) {
 			TAILQ_REMOVE(&st->st_entry, se, se_list);
@@ -1357,7 +1369,7 @@ adhoc_age(struct ieee80211_scan_state *ss)
 			FREE(se, M_80211_SCAN);
 		}
 	}
-	spin_unlock(&st->st_lock);
+	SCAN_STA_UNLOCK_IRQ(st);
 }
 
 /*
@@ -1374,8 +1386,8 @@ adhoc_default_action(struct ieee80211vap *vap,
 	if (IEEE80211_ADDR_EQ(se->se_bssid, &zeroMacAddr[0])) {
 		ieee80211_create_ibss(vap, se->se_chan);
 		return 1;
-	} else 
-		return ieee80211_sta_join(vap,se);
+	} else
+		return ieee80211_sta_join(vap, se);
 }
 
 static const struct ieee80211_scanner adhoc_default = {
@@ -1413,7 +1425,7 @@ action_tasklet(IEEE80211_TQUEUE_ARG data)
 		break;
 	default:
 		/* ADHOC */
-        	if (vap->iv_des_nssid) {
+		if (vap->iv_des_nssid) {
 			/*
 			 * No existing adhoc network to join and we have
 			 * an ssid; start one up.  If no channel was
@@ -1425,7 +1437,7 @@ action_tasklet(IEEE80211_TQUEUE_ARG data)
 				chan = vap->iv_des_chan;
 			if (chan != NULL) {
 				struct ieee80211_scan_entry se;
-				
+
 				memset(&se, 0, sizeof(se));
 				se.se_chan = chan;
 				if ((*ss->ss_ops->scan_default)(vap, &se))
@@ -1434,14 +1446,14 @@ action_tasklet(IEEE80211_TQUEUE_ARG data)
 		}
 		/*
 		 * If nothing suitable was found decrement
-	         * the failure counts so entries will be
+		 * the failure counts so entries will be
 		 * reconsidered the next time around.  We
-		 * really want to do this only for sta's
+		 * really want to do this only for STAs
 		 * where we've previously had some success.
 		 */
 		sta_dec_fails(st);
 		st->st_newscan = 1;
-			break;
+		break;
 	}
 
 	/* 
