@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: test_ccmp.c 1721 2006-09-20 08:45:13Z mentor $
+ * $Id: test_ccmp.c 2902 2007-11-21 20:14:11Z mtaylor $
  */
 
 /*
@@ -41,7 +41,7 @@
  * module.  It should automatically run all test cases and print
  * information for each.  To run one or more tests you can specify a
  * tests parameter to the module that is a bit mask of the set of tests
- * you want; e.g. insmod ccmp_test tests=7 will run only test mpdu's
+ * you want; e.g. insmod ccmp_test tests=7 will run only test MPDUs
  * 1, 2, and 3.
  */
 #ifndef AUTOCONF_INCLUDED
@@ -99,10 +99,10 @@ static const u_int8_t test1_key[] = {		/* TK */
 static const u_int8_t test1_plaintext[] = {	/* Plaintext MPDU w/o MIC */
 	0x08, 0x48, 0xc3, 0x2c, 0x0f, 0xd2, 0xe1, 0x28,	/* 802.11 Header */
 	0xa5, 0x7c, 0x50, 0x30, 0xf1, 0x84, 0x44, 0x08,
-	0xab, 0xae, 0xa5, 0xb8, 0xfc, 0xba, 0x80, 0x33, 
+	0xab, 0xae, 0xa5, 0xb8, 0xfc, 0xba, 0x80, 0x33,
 	0xf8, 0xba, 0x1a, 0x55, 0xd0, 0x2f, 0x85, 0xae,	/* Plaintext Data */
 	0x96, 0x7b, 0xb6, 0x2f, 0xb6, 0xcd, 0xa8, 0xeb,
-	0x7e, 0x78, 0xa0, 0x50, 
+	0x7e, 0x78, 0xa0, 0x50,
 };
 static const u_int8_t test1_encrypted[] = {	/* Encrypted MPDU with MIC */
 	0x08, 0x48, 0xc3, 0x2c, 0x0f, 0xd2, 0xe1, 0x28,
@@ -612,7 +612,7 @@ runtest(struct ieee80211com *ic, struct ciphertest *t)
 	 * Craft frame from plaintext data.
 	 */
 	cip = key.wk_cipher;
-	skb = dev_alloc_skb(t->plaintext_len +
+	skb = ieee80211_dev_alloc_skb(t->plaintext_len +
 		cip->ic_header + cip->ic_trailer);
 	if (skb == NULL) {
 		printk("FAIL: unable to allocate skbuff\n");
@@ -667,13 +667,13 @@ runtest(struct ieee80211com *ic, struct ciphertest *t)
 			t->plaintext, sizeof(t->plaintext));
 		goto bad;
 	}
-	dev_kfree_skb(skb);
+	ieee80211_dev_kfree_skb(&skb);
 	ieee80211_crypto_delkey(ic, &key);
 	printk("PASS\n");
 	return 1;
 bad:
 	if (skb != NULL)
-		dev_kfree_skb(skb);
+		ieee80211_dev_kfree_skb(&skb);
 	ieee80211_crypto_delkey(ic, &key);
 	return 0;
 }
